from dexy.plugins.process_filters import SubprocessFilter
from dexy.plugins.process_filters import SubprocessStdoutFilter
import os

class CasperJsSvg2PdfFilter(SubprocessFilter):
    """
    Converts an SVG file to PDF by running it through casper js.
    # TODO convert this to phantomjs, no benefit to using casper here (js is not user facing) and more restrictive
    """
    ADD_NEW_FILES = True
    ALIASES = ['svg2pdf']
    EXECUTABLE = 'casperjs'
    INPUT_EXTENSIONS = ['.svg']
    OUTPUT_EXTENSIONS = ['.pdf']
    VERSION_COMMAND = 'casperjs --version'

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or ""
        }
        return "%(prog)s %(args)s script.js" % args

    def script_js(self, width, height):
        args = {
                'width' : width,
                'height' : height,
                'svgfile' : self.input_filename(),
                'pdffile' : self.output_filename()
                }
        return """
        var casper = require('casper').create({
             viewportSize : {width : %(width)s, height : %(height)s}
        });
        casper.start('%(svgfile)s', function() {
            this.capture('%(pdffile)s');
        });

        casper.run();
        """ % args

    def setup_wd(self):
        wd = self.artifact.working_dir()
        if not os.path.exists(wd):
            for doc, filename in self.artifact.setup_wd(self.input_filename()):
                self.write_to_wd(wd, doc, filename)

        width = self.args().get('width', 200)
        height = self.args().get('height', 200)
        js = self.script_js(width, height)

        wd = os.path.join(self.artifact.tmp_dir(), self.input().parent_dir())
        scriptfile = os.path.join(wd, "script.js")
        self.log.debug("scriptfile: %s" % scriptfile)
        with open(scriptfile, "w") as f:
            f.write(js)

        return wd

class CasperJsStdoutFilter(SubprocessStdoutFilter):
    """
    Runs scripts using casper js. Saves cookies.
    """
    ALIASES = ['casperjs']
    ADD_NEW_FILES = True
    EXECUTABLE = 'casperjs'
    INPUT_EXTENSIONS = ['.js', '.txt']
    OUTPUT_EXTENSIONS = ['.txt']
    VERSION_COMMAND = 'casperjs --version'

    def command_string_stdout(self):
        args = {
            'cookie_file' : 'cookies.txt',
            'prog' : self.executable(),
            'args' : self.command_line_args() or "",
            'scriptargs' : self.command_line_scriptargs() or "",
            'script_file' : self.input_filename()
        }
        return "%(prog)s --cookies-file=%(cookie_file)s %(args)s %(script_file)s %(scriptargs)s" % args

class PhantomJsStdoutFilter(SubprocessStdoutFilter):
    """
    Runs scripts using phantom js.
    """
    ADD_NEW_FILES = True
    ALIASES = ['phantomjs']
    EXECUTABLE = 'phantomjs'
    INPUT_EXTENSIONS = ['.js', '.txt']
    OUTPUT_EXTENSIONS = ['.txt']
    VERSION_COMMAND = 'phantomjs --version'
    # TODO ensure phantom.exit() is called in script?

class PhantomJsRenderSubprocessFilter(SubprocessFilter):
    """
    Renders HTML to PNG/PDF using phantom.js. If the HTML relies on local
    assets such as CSS or image files, these should be specified as inputs.
    """
    ADD_NEW_FILES = True
    ALIASES = ['phrender']
    EXECUTABLE = 'phantomjs'
    INPUT_EXTENSIONS = [".html", ".htm", ".txt"]
    OUTPUT_EXTENSIONS = [".png", ".pdf"]
    VERSION_COMMAND = 'phantomjs --version'
    DEFAULT_WIDTH = 1024
    DEFAULT_HEIGHT = 768

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or ""
        }
        return "%(prog)s %(args)s script.js" % args

    def setup_wd(self):
        wd = self.artifact.working_dir()
        if not os.path.exists(wd):
            for doc, filename in self.artifact.setup_wd(self.input_filename()):
                self.write_to_wd(wd, doc, filename)

        width = self.arg_value('width', self.DEFAULT_WIDTH)
        height = self.arg_value('height', self.DEFAULT_HEIGHT)

        timeout = self.setup_timeout()
        if not timeout:
            timeout = 200

        args = {
                'address' : self.input_filename(),
                'output' : self.output_filename(),
                'width' : width,
                'height' : height,
                'timeout' : timeout
                }

        js = """
        address = '%(address)s'
        output = '%(output)s'
        var page = new WebPage(),
            address, output, size;

        page.viewportSize = { width: %(width)s, height: %(height)s };
        page.open(address, function (status) {
            if (status !== 'success') {
                console.log('Unable to load the address!');
            } else {
                window.setTimeout(function () {
                page.render(output);
                phantom.exit();
                }, %(timeout)s);
            }
        });
        """ % args

        scriptfile = os.path.join(wd, "script.js")
        self.log.debug("scriptfile: %s" % scriptfile)
        with open(scriptfile, "w") as f:
            f.write(js)

        return wd



from dexy.plugins.process_filters import SubprocessStdoutFilter
from dexy.plugins.process_filters import SubprocessInputFileFilter
import json

class NodeJsStdoutFilter(SubprocessStdoutFilter):
    """
    Runs scripts using node js.
    """
    ADD_NEW_FILES = True
    ALIASES = ['nodejs', 'node']
    EXECUTABLE = 'node'
    INPUT_EXTENSIONS = ['.js', '.txt']
    OUTPUT_EXTENSIONS = ['.txt']
    VERSION_COMMAND = 'node --version'

class Regetron(SubprocessInputFileFilter):
    """
    Filter which loads .regex file into regetron and runs any input text against it.
    """
    ALIASES = ['regetron']
    EXECUTABLE = 'regetron'
    INPUT_EXTENSIONS = [".regex"]
    OUTPUT_EXTENSIONS = [".txt"]

    def command_string_for_input(self, input_doc):
        text = input_doc.output().name
        regex = self.input_filename()
        return "%s %s %s" % (self.executable(), text, regex)

class Pdfinfo(SubprocessStdoutFilter):
    """
    Uses the pdfinfo script to retrieve metadata about a PDF.
    """
    ALIASES = ['pdfinfo']
    EXECUTABLE = 'pdfinfo'
    INPUT_EXTENSIONS = ['.pdf']
    OUTPUT_EXTENSIONS = ['.txt']

class Wordcount(SubprocessStdoutFilter):
    """
    Runs input through wc command line tool.
    """
    ALIASES = ['wc']
    EXECUTABLE = 'wc'
    OUTPUT_EXTENSIONS = ['.txt']

class Python(SubprocessStdoutFilter):
    """
    Runs Python code and returns stdout.
    """
    ALIASES = ['py', 'pyout']
    EXECUTABLE = 'python'
    INPUT_EXTENSIONS = [".py", ".txt"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION_COMMAND = 'python --version'
    TAGS = ['python']

class Bash(SubprocessStdoutFilter):
    """
    Runs bash scripts and returns stdout.
    """
    ALIASES = ['sh', 'bash']
    EXECUTABLE = 'bash -e'
    INPUT_EXTENSIONS = [".sh", ".bash", ".txt", ""]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION_COMMAND = 'bash --version'

class Ruby(SubprocessStdoutFilter):
    """
    Runs ruby scripts and return stdout.
    """
    EXECUTABLE = 'ruby'
    VERSION_COMMAND = 'ruby --version'
    INPUT_EXTENSIONS = [".txt", ".rb"]
    OUTPUT_EXTENSIONS = [".txt"]
    ALIASES = ['rb']

class Irb(SubprocessStdoutFilter):
    """
    Runs ruby scripts in irb.
    """
    ALIASES = ['irbout']
    CHECK_RETURN_CODE = False
    EXECUTABLE = 'irb --simple-prompt --noreadline'
    INPUT_EXTENSIONS = [".txt", ".rb"]
    OUTPUT_EXTENSIONS = [".rbcon"]
    VERSION_COMMAND = 'irb --version'

class Sloccount(SubprocessStdoutFilter):
    """
    Runs code through sloccount.
    """
    EXECUTABLE = 'sloccount'
    VERSION_COMMAND = 'sloccount --version'
    INPUT_EXTENSIONS = [".*"]
    OUTPUT_EXTENSIONS = [".txt"]
    ALIASES = ['sloc', 'sloccount']

class Rhino(SubprocessStdoutFilter):
    """
    Runs code through rhino js interpreter.
    """
    EXECUTABLE = "rhino -f"
    INPUT_EXTENSIONS = [".js", ".txt"]
    OUTPUT_EXTENSIONS = [".txt"]
    ALIASES = ['js', 'rhino']

class Lua(SubprocessStdoutFilter):
    """
    Runs code through lua interpreter.
    """
    EXECUTABLE = 'lua'
    VERSION_COMMAND = 'lua -v'
    INPUT_EXTENSIONS = ['.lua', '.txt']
    OUTPUT_EXTENSIONS = ['.txt']
    ALIASES = ['lua']

class Cowsay(SubprocessStdoutFilter):
    """
    Runs input through 'cowsay'.
    """
    ADD_NEW_FILES = False
    ALIASES = ['cowsay']
    EXECUTABLE = 'cowsay'
    INPUT_EXTENSIONS = [".txt"]
    OUTPUT_EXTENSIONS = [".txt"]

    def command_string_stdout(self):
        args = self.command_line_args() or ""
        text = self.input().as_text()
        return "%s %s \"%s\"" % (self.executable(), args, text)

class Cowthink(Cowsay):
    """
    Runs input through 'cowthink'.
    """
    ALIASES = ['cowthink']
    EXECUTABLE = 'cowthink'

class Figlet(Cowsay):
    """
    Runs input through 'figlet'.
    """
    ALIASES = ['figlet']
    EXECUTABLE = 'figlet'

class Wiki2Beamer(SubprocessStdoutFilter):
    """
    Converts wiki content to beamer.
    """
    ALIASES = ['wiki2beamer']
    EXECUTABLES = ['wiki2beamer']
    INPUT_EXTENSIONS = [".wiki", ".txt"]
    OUTPUT_EXTENSIONS = [".tex"]
    VERSION_COMMAND = "wiki2beamer --version"

class Strings(SubprocessStdoutFilter):
    """
    Clean non-printing characters from text using the 'strings' tool.
    """
    ALIASES = ['strings']
    EXECUTABLE = 'strings'

class Php(SubprocessStdoutFilter):
    """
    Runs php file, note that php code must be included in <?php ... ?> tags.
    """
    EXECUTABLE = 'php'
    CHECK_RETURN_CODE = False # TODO see if it's possible to check return code for PHP
    VERSION_COMMAND = 'php --version'
    INPUT_EXTENSIONS = [".php", ".txt"]
    OUTPUT_EXTENSIONS = [".html", ".txt"]
    ALIASES = ['php']

class RagelRubyDot(SubprocessStdoutFilter):
    """
    Generates state chart in .dot format of ragel state machine for ruby.
    """
    INPUT_EXTENSIONS = [".rl"]
    OUTPUT_EXTENSIONS = [".dot"]
    ALIASES = ['rlrbd', 'ragelrubydot']
    VERSION_COMMAND = 'ragel --version'
    EXECUTABLE = 'ragel -R -V'

class ManPage(SubprocessStdoutFilter):
    """
    Read command names from a file and fetch man pages for each.

    Returns a JSON dict whose keys are the program names and values are man
    pages.
    """
    ALIASES = ['man']
    EXECUTABLE = 'man'
    REQUIRED_EXECUTABLES = ['strings']
    VERSION_COMMAND = 'man --version'
    INPUT_EXTENSIONS = [".txt"]
    OUTPUT_EXTENSIONS = [".json"]

    def command_string(self, prog_name):
        # Use bash rather than the default of sh (dash) so we can set pipefail.
        return "bash -c \"set -e; set -o pipefail; man %s | col -b | strings\"" % (prog_name)

    def process(self):
        man_info = {}
        for prog_name in self.input().data().split():
            command = self.command_string(prog_name)
            proc, stdout = self.run_command(command, self.setup_env())
            self.handle_subprocess_proc_return(command, proc.returncode, stdout)
            man_info[prog_name] = stdout

        self.output().set_data(json.dumps(man_info))

class Lynxdump(SubprocessStdoutFilter):
    """
    Converts HTML to plain text by using lynx -dump.
    """
    INPUT_EXTENSIONS = [".html"]
    OUTPUT_EXTENSIONS = [".txt"]
    ALIASES = ['lynxdump']
    VERSION_COMMAND = 'lynx --version'
    EXECUTABLE = 'lynx -dump'

class Escript(SubprocessStdoutFilter):
    """
    Runs Erlang scripts using the escript command.
    """
    EXECUTABLE = 'escript'
    INPUT_EXTENSIONS = [".erl"]
    OUTPUT_EXTENSIONS = [".txt"]
    ALIASES = ['escript']

class Redcloth(SubprocessStdoutFilter):
    """
    Runs redcloth, converts textile markup to HTML.
    """
    EXECUTABLE = 'redcloth'
    VERSION_COMMAND = 'redcloth --version'
    INPUT_EXTENSIONS = [".txt", ".textile"]
    OUTPUT_EXTENSIONS = [".html"]
    ALIASES = ['redcloth', 'textile']

class RedclothLatex(SubprocessStdoutFilter):
    """
    Runs redcloth, converts textile markup to LaTeX.
    """
    EXECUTABLE = 'redcloth -o latex'
    VERSION_COMMAND = 'redcloth --version'
    INPUT_EXTENSIONS = [".txt", ".textile"]
    OUTPUT_EXTENSIONS = [".tex"]
    ALIASES = ['redclothl', 'latextile']



from dexy.filter import DexyFilter

class PreserveDataClassFilter(DexyFilter):
    """
    Sets PRESERVE_PRIOR_DATA_CLASS to True.
    """
    ALIASES = []
    PRESERVE_PRIOR_DATA_CLASS = True

    def calculate_canonical_name(self):
        return self.artifact.prior.filter_instance.calculate_canonical_name()

class ChangeExtensionManuallyFilter(PreserveDataClassFilter):
    """
    Dummy filter for allowing changing a file extension.
    """
    ALIASES = ['chext']

class ForceCPickleExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .cpickle extension.
    """
    INPUT_EXTENSIONS = [".cpickle"]
    OUTPUT_EXTENSIONS = [".cpickle"]
    ALIASES = ['cpickle']

class ForcePickleExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .pickle extension.
    """
    INPUT_EXTENSIONS = [".pickle"]
    OUTPUT_EXTENSIONS = [".pickle"]
    ALIASES = ['pickle']

class ForceHtmlExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .html extension.
    """
    INPUT_EXTENSIONS = [".html"]
    OUTPUT_EXTENSIONS = [".html"]
    ALIASES = ['h', 'forcehtml']

class ForceJsonExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .json extension.
    """
    INPUT_EXTENSIONS = [".json"]
    OUTPUT_EXTENSIONS = [".json"]
    ALIASES = ['j', 'forcejson']

class ForceSvgExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .svg extension.
    """
    INPUT_EXTENSIONS = [".svg"]
    OUTPUT_EXTENSIONS = [".svg"]
    ALIASES = ['svg', 'forcesvg']

class ForceXmlExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .xml extension.
    """
    INPUT_EXTENSIONS = [".xml"]
    OUTPUT_EXTENSIONS = [".xml"]
    ALIASES = ['x', 'forcexml']

class ForceLatexExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .tex extension.
    """
    INPUT_EXTENSIONS = [".tex"]
    OUTPUT_EXTENSIONS = [".tex"]
    ALIASES = ['l', 'forcelatex']

class ForceTextExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .txt extension.
    """
    INPUT_EXTENSIONS = [".txt"]
    OUTPUT_EXTENSIONS = [".txt"]
    ALIASES = ['t', 'forcetext']

class ForceRExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .R extension.
    """
    INPUT_EXTENSIONS = [".R"]
    OUTPUT_EXTENSIONS = [".R"]
    ALIASES = ['forcer']

class ForcePdfExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .pdf extension.
    """
    INPUT_EXTENSIONS = [".pdf"]
    OUTPUT_EXTENSIONS = [".pdf"]
    ALIASES = ['p', 'forcepdf']

class ForceJpgExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .jpg extension.
    """
    INPUT_EXTENSIONS = [".jpg"]
    OUTPUT_EXTENSIONS = [".jpg"]
    ALIASES = ['jn', 'forcejpg']

class ForcePngExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .png extension.
    """
    INPUT_EXTENSIONS = [".png"]
    OUTPUT_EXTENSIONS = [".png"]
    ALIASES = ['pn', 'forcepng']

class ForceGifExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .gif extension.
    """
    INPUT_EXTENSIONS = [".gif"]
    OUTPUT_EXTENSIONS = [".gif"]
    ALIASES = ['gn', 'forcegif']

class ForceBmpExtensionFilter(PreserveDataClassFilter):
    """
    Forces previous filter to output .bmp extension.
    """
    INPUT_EXTENSIONS = [".bmp"]
    OUTPUT_EXTENSIONS = [".bmp"]
    ALIASES = ['bn', 'forcebmp']

class ConvertBashExtensionFilter(PreserveDataClassFilter):
    """
    Changes file extension to .sh
    """
    INPUT_EXTENSIONS = [".*", "*"]
    OUTPUT_EXTENSIONS = [".sh"]
    ALIASES = ['cb']

class ConvertTextExtensionFilter(PreserveDataClassFilter):
    """
    Changes file extension to .txt
    """
    INPUT_EXTENSIONS = [".*"]
    OUTPUT_EXTENSIONS = [".txt"]
    ALIASES = ['ct']

class ConvertHTMLExtensionFilter(PreserveDataClassFilter):
    """
    Changes file extension to .html
    """
    INPUT_EXTENSIONS = [".*"]
    OUTPUT_EXTENSIONS = [".html"]
    ALIASES = ['ch']

class ConvertJsonExtensionFilter(PreserveDataClassFilter):
    """
    Changes file extension to .json
    """
    INPUT_EXTENSIONS = [".*"]
    OUTPUT_EXTENSIONS = [".json"]
    ALIASES = ['cj']



from dexy.common import OrderedDict
from dexy.node import DocNode
from dexy.tests.utils import wrap

REGETRON_INPUT_1 = "hello\n"
REGETRON_INPUT_2 = """\
this is some text
9
nine
this is 100 mixed text and numbers
"""

def test_regetron_filter():
    with wrap() as wrapper:
        node = DocNode("example.regex|regetron",
                inputs = [
                    DocNode("input1.txt",
                        contents=REGETRON_INPUT_1,
                        wrapper=wrapper),
                    DocNode("input2.txt",
                        contents=REGETRON_INPUT_2,
                        wrapper=wrapper)
                    ],
                contents="^[a-z\s]+$",
                wrapper=wrapper)

        wrapper.run_docs(node)
        doc = node.children[0]

        assert doc.output()['input1.txt'] == """\
> ^[a-z\s]+$
0000: hello
> 

"""
        assert doc.output()['input2.txt'] == """\
> ^[a-z\s]+$
0000: this is some text
0002: nine
> 

"""

def test_used_filter():
    with wrap() as wrapper:
        node = DocNode("input.txt|used",
                inputs = [
                    DocNode("example.sed",
                        contents="s/e/E/g",
                        wrapper=wrapper)
                    ],
                contents="hello",
                wrapper=wrapper)

        wrapper.run_docs(node)
        doc = node.children[0]
        assert str(doc.output()) == "hEllo"

def test_sed_filter_single_simple_input_file():
    with wrap() as wrapper:
        node = DocNode("example.sed|sed",
                inputs = [
                    DocNode("input.txt",
                        contents="hello",
                        wrapper=wrapper)
                    ],
                contents="s/e/E/g",
                wrapper=wrapper)

        wrapper.run_docs(node)
        doc = node.children[0]
        assert str(doc.output()) == "hEllo"

def test_sed_filter_single_input_file_with_sections():
    with wrap() as wrapper:
        node = DocNode("example.sed|sed",
                contents="s/e/E/g",
                inputs = [
                    DocNode("input.txt",
                        contents=OrderedDict([
                            ('foo', 'hello'),
                            ('bar', 'telephone')
                            ]),
                        wrapper=wrapper)
                        ],
                wrapper=wrapper)

        wrapper.run_docs(node)
        doc = node.children[0]
        assert doc.output()['foo'] == 'hEllo'
        assert doc.output()['bar'] == 'tElEphonE'

def test_sed_filter_multiple_inputs():
    with wrap() as wrapper:
        node = DocNode("example.sed|sed",
                contents="s/e/E/g",
                inputs = [
                    DocNode("foo.txt",
                        contents='hello',
                        wrapper=wrapper),
                    DocNode("bar.txt",
                        contents='telephone',
                        wrapper=wrapper)
                    ],
                wrapper=wrapper)

        wrapper.run_docs(node)
        doc = node.children[0]
        assert doc.output()['foo.txt'] == 'hEllo'
        assert doc.output()['bar.txt'] == 'tElEphonE'



from dexy.common import OrderedDict
from dexy.plugins.process_filters import SubprocessFilter
from dexy.plugins.process_filters import SubprocessInputFilter
import dexy.exceptions
import json
import os
import shutil

class PythonInput(SubprocessInputFilter):
    """
    Runs python code and passes input.
    """
    EXECUTABLE = 'python'
    ALIASES = ['pyin']
    VERSION_COMMAND = 'python --version'
    OUTPUT_EXTENSIONS = ['.txt']

class Pdfcrop(SubprocessFilter):
    """
    Runs the PDFcrop script http://pdfcrop.sourceforge.net/
    """
    ALIASES = ['pdfcrop']
    EXECUTABLE = 'pdfcrop'
    INPUT_EXTENSIONS = ['.pdf']
    OUTPUT_EXTENSIONS = ['.pdf']

class CalibreFilter(SubprocessFilter):
    """
    Invokes ebook-convert command line tool (part of calibre) to generate various output formats (including .mobi for Kindle)

    http://manual.calibre-ebook.com/cli/ebook-convert.html
    """
    ADD_NEW_FILES = False
    ALIASES = ['calibre', 'ebook']
    EXECUTABLE = "ebook-convert"
    FRAGMENT = False
    INPUT_EXTENSIONS = ['.html', '.epub', '.azw', '.chm', '.comic', '.djvu',
            '.pdf', '.mobi', '.lit', '.fb2']
    OUTPUT_EXTENSIONS = ['.mobi', '.epub', '.fb2', '.htmlz', '.lit', '.lrf',
            '.pdf', '.rtf', '.snb', '.tcr', '.txt', '.txtz', '.html', '.pml']
    PATH_EXTENSIONS = ['/Applications/calibre.app/Contents/MacOS']
    VERSION_COMMAND = "ebook-convert --version"

#    TODO: output oeb (outputs a directory)
#    TODO: output azw
#    TODO: test with html, pml plugins

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or "",
            'input_file' : self.input_filename(),
            'output_file' : self.output_filename()
        }
        return """%(prog)s "%(input_file)s" "%(output_file)s" %(args)s""" % args

class LyxFilter(SubprocessFilter):
    """
    Invokes lyx on the command line. By default generates a latex file.
    """
    EXECUTABLE = 'lyx'
    ALIASES = ['lyx']
    VERSION_COMMAND = 'lyx -version'
    OUTPUT_EXTENSIONS = ['.tex', '.pdf', '.lyx']

    def command_string(self):
        cl_args = self.command_line_args()
        if cl_args and ("-e" in cl_args):
            format_string = ''
        else:
            format_string = "-e latex"

        args = {
            'format_string' : format_string,
            'prog' : self.executable(),
            'args' : self.command_line_args() or "",
            'input_file' : self.input_filename(),
        }
        return "%(prog)s %(format_string)s %(args)s %(input_file)s" % args

class PandocFilter(SubprocessFilter):
    """
    Converts documents using pandoc.
    """
    EXECUTABLE = "pandoc"
    VERSION_COMMAND = "pandoc --version"
    ALIASES = ['pandoc']
    OUTPUT_EXTENSIONS = ['.html', '.txt', '.tex', '.pdf', '.rtf', '.json', '.docx', '.odt', '.epub']
    FRAGMENT = False
    ADD_NEW_FILES = False

    # TODO Support all these + test them.
    #Output formats: native, json, html, html5, html+lhs, html5+lhs, s5, slidy,
    #                dzslides, docbook, opendocument, latex, latex+lhs, beamer,
    #                context, texinfo, man, markdown, markdown+lhs, plain, rst,
    #                rst+lhs, mediawiki, textile, rtf, org, asciidoc, odt, docx,
    #                epub

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or "",
            'script_file' : self.input_filename(),
            'output_file' : self.output_filename()
        }
        return "%(prog)s %(args)s %(script_file)s -o %(output_file)s" % args

class EspeakFilter(SubprocessFilter):
    """
    Runs espeak text to speech.
    """
    EXECUTABLE = "espeak"
    INP2UT_EXTENSIONS = [".txt"]
    OUTPUT_EXTENSIONS = [".wav"]
    ALIASES = ['espeak']

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or "",
            'scriptargs' : self.command_line_scriptargs() or "",
            'script_file' : self.input_filename(),
            'output_file' : self.output_filename()
        }
        return "%(prog)s %(args)s -w %(output_file)s %(script_file)s" % args

class AsciidocFilter(SubprocessFilter):
    """
    Converts ASCIIDOC input.
    """
    VERSION_COMMAND = "asciidoc --version"
    EXECUTABLE = "asciidoc"
    INPUT_EXTENSIONS = [".txt"]
    OUTPUT_EXTENSIONS = [".html", ".xml"]
    ALIASES = ['asciidoc']

    def command_string(self):
        if self.args().has_key('backend'):
            backend = self.arg_value('backend')
            # TODO check file extension is valid for backend
        else:
            if self.artifact.ext == ".html":
                if self.version() >= "asciidoc 8.6.5":
                    backend = 'html5'
                else:
                    backend = 'html4'
            elif self.artifact.ext == ".xml":
                backend = "docbook45"
            elif self.artifact.ext == ".tex":
                backend = "latex"
            else:
                raise Exception("unexpected file extension in asciidoc filter %s" % self.artifact.ext)

        args = {
            'backend' : backend,
            'infile' : self.input_filename(),
            'outfile' : self.output_filename(),
            'prog' : self.executable(),
            'args' : self.command_line_args() or ""
        }

        return "%(prog)s -b %(backend)s %(args)s -o %(outfile)s %(infile)s" % args

class BlackWhitePdfFilter(SubprocessFilter):
    """
    Converts color pdf to black and white.
    """
    EXECUTABLE = "gs"
    ALIASES = ['bw', 'bwconv']
    INPUT_EXTENSIONS = [".pdf"]
    OUTPUT_EXTENSIONS = [".pdf"]

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'in' : self.input_filename(),
            'out' : self.output_filename()
        }
        s = "%(prog)s -dSAFER -dNOPAUSE -dBATCH -sDEVICE=pdfwrite -sColorConversionStrategy=Gray -dProcessColorModel=/DeviceGray -sOutputFile=%(out)s %(in)s"
        return s % args

class Pdf2ImgSubprocessFilter(SubprocessFilter):
    """
    Converts a PDF file to a PNG image using ghostscript.

    Returns the image generated by page 1 of the PDF by default, the
    'page' parameter can be used to specify other pages.
    """
    ALIASES = ['pdf2img', 'pdf2png']
    EXECUTABLE = "gs"
    GS_DEVICE = 'png16m -r300'
    INPUT_EXTENSIONS = ['.pdf']
    OUTPUT_EXTENSIONS = ['.png']
    VERSION_COMMAND = "gs --version"

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'device' : self.GS_DEVICE,
            'in' : self.input_filename(),
            'out' : self.output_filename()
        }
        s = "%(prog)s -dSAFER -dNOPAUSE -dBATCH -sDEVICE=%(device)s -sOutputFile=%%d-%(out)s %(in)s"
        return s % args

    def process(self):
        command = self.command_string()
        proc, stdout = self.run_command(command, self.setup_env())
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)


        page = self.args().get('page', 1)
        page_file = "%s-%s" % (page, self.output().basename())

        wd = self.setup_wd()
        page_path = os.path.join(wd, page_file)
        shutil.copyfile(page_path, self.output_filepath())

class Pdf2JpgSubprocessFilter(Pdf2ImgSubprocessFilter):
    """
    Converts a PDF file to a jpg image using ghostscript.

    Returns the image generated by page 1 of the PDF by default, the
    'page' parameter can be used to specify other pages.
    """
    ALIASES = ['pdf2jpg']
    GS_DEVICE = 'jpeg'
    OUTPUT_EXTENSIONS = ['.jpg']

class DotFilter(SubprocessFilter):
    """
    Renders .dot files to either PNG or PDF images.
    """
    INPUT_EXTENSIONS = [".dot"]
    OUTPUT_EXTENSIONS = [".png", ".pdf"]
    EXECUTABLE = 'dot'
    VERSION_COMMAND = 'dot -V'
    ALIASES = ['dot', 'graphviz']

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'format' : self.artifact.ext.replace(".",""),
            'workfile' : self.input_filename(),
            'outfile' : self.output_filename()
        }
        return "%(prog)s -T%(format)s -o%(outfile)s %(workfile)s" % args

class Html2PdfSubprocessFilter(SubprocessFilter):
    """
    Renders HTML to PDF using wkhtmltopdf. If the HTML relies on assets such as
    CSS or image files, these should be specified as inputs.

    If you have an older version of wkhtmltopdf, and are running on a server,
    you may get XServer errors. You can install xvfb and run Dexy as
    "xvfb-run dexy". Or upgrade to the most recent wkhtmltopdf which only needs
    X11 client libs.
    """
    ALIASES = ['html2pdf', 'wkhtmltopdf']
    EXECUTABLE = 'wkhtmltopdf'
    INPUT_EXTENSIONS = [".html", ".txt"]
    OUTPUT_EXTENSIONS = [".pdf"]
    VERSION_COMMAND = 'wkhtmltopdf --version'

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'in' : self.input_filename(),
            'out' : self.output_filename()
        }
        return "%(prog)s %(in)s %(out)s" % args

class Ps2PdfSubprocessFilter(SubprocessFilter):
    """
    Converts a postscript file to PDF format.
    """
    ALIASES = ['ps2pdf']
    EXECUTABLE = 'ps2pdf'
    INPUT_EXTENSIONS = [".ps", ".txt"]
    OUTPUT_EXTENSIONS = [".pdf"]

class RagelRubySubprocessFilter(SubprocessFilter):
    """
    Generates ruby source code from a ragel file.
    """
    ALIASES = ['rlrb', 'ragelruby']
    EXECUTABLE = 'ragel -R'
    INPUT_EXTENSIONS = [".rl"]
    OUTPUT_EXTENSIONS = [".rb"]
    VERSION_COMMAND = 'ragel --version'

    def command_string(self):
        wf = self.input_filename()
        of = self.output_filename()
        return "%s %s -o %s" % (self.executable(), wf, of)

class Rd2PdfFilter(SubprocessFilter):
    """
    Generates a pdf from R documentation file.
    """
    INPUT_EXTENSIONS = [".Rd"]
    OUTPUT_EXTENSIONS = [".pdf", ".dvi"]
    EXECUTABLE = 'R CMD Rd2pdf'
    VERSION_COMMAND = 'R CMD Rd2pdf -v'
    ALIASES = ['rd2pdf', 'Rd2pdf']

    def command_string(self):
        title = self.baserootname().replace("_", " ")
        args = {
            'prog' : self.executable(),
            'out' : self.output_filename(),
            'in' : self.input_filename(),
            'title' : title
        }
        return "%(prog)s --output=%(out)s --title=\"%(title)s\" %(in)s" % args

class RBatchFilter(SubprocessFilter):
    """Runs R code in batch mode."""
    ADD_NEW_FILES = True
    ALIASES = ['rintbatch']
    EXECUTABLE = 'R CMD BATCH --quiet --no-timing'
    INPUT_EXTENSIONS = ['.txt', '.r', '.R']
    OUTPUT_EXTENSIONS = [".Rout", '.txt']
    VERSION_COMMAND = "R --version"

class RIntBatchSectionsFilter(SubprocessFilter):
    """
    Experimental filter to run R in sections without using pexpect.
    """
    ADD_NEW_FILES = True
    ALIASES = ['rintmock']
    EXECUTABLE = 'R CMD BATCH --quiet --no-timing'
    INPUT_EXTENSIONS = ['.txt', '.r', '.R']
    OUTPUT_EXTENSIONS = [".Rout", '.txt']
    VERSION_COMMAND = "R --version"
    WRITE_STDERR_TO_STDOUT = False
    OUTPUT_DATA_TYPE = 'sectioned'

    def command_string(self, section_name, section_text, wd):
        br = self.input().baserootname()
        work_file = "%s-%s%s" % (br, section_name, self.input().ext)
        outfile = "%s-%s-out%s" % (br, section_name, self.output().ext)

        work_filepath = os.path.join(wd, work_file)

        with open(work_filepath, "wb") as f:
            f.write(section_text)

        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or "",
            'script_file' : work_file,
            'scriptargs' : self.command_line_scriptargs() or "",
            'output_file' : outfile
        }

        command = """%(prog)s %(args)s "%(script_file)s" %(scriptargs)s "%(output_file)s" """ % args
        return command, outfile

    def process(self):
        wd = self.setup_wd()

        result = OrderedDict()

        for section_name, section_text in self.input().as_sectioned().iteritems():
            command, outfile = self.command_string(section_name, section_text, wd)
            proc, stdout = self.run_command(command, self.setup_env())
            self.handle_subprocess_proc_return(command, proc.returncode, stdout)

            with open(os.path.join(wd, outfile), "rb") as f:
                result[section_name] = f.read()

        if self.do_walk_working_directory():
            self.walk_working_directory()

        if self.do_add_new_files():
            self.add_new_files()

        self.output().set_data(result)

class ROutputBatchFilter(SubprocessFilter):
    """Runs R code in batch mode. Uses the --slave flag so doesn't echo commands, just returns output."""
    ADD_NEW_FILES = True
    ALIASES = ['rout', 'routbatch']
    EXECUTABLE = 'R CMD BATCH --vanilla --quiet --slave --no-timing'
    INPUT_EXTENSIONS = ['.R', '.r', '.txt']
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION_COMMAND = "R --version"

class EmbedFonts(SubprocessFilter):
    """
    Use to embed fonts and do other prepress as required for some types of printing.
    """
    INPUT_EXTENSIONS = [".pdf"]
    OUTPUT_EXTENSIONS = [".pdf"]
    EXECUTABLE = 'ps2pdf'
    ALIASES = ['embedfonts', 'prepress']

    def preprocess_command_string(self):
        pf = self.input_filename()
        af = self.output_filename()
        return "%s -dPDFSETTINGS=/prepress %s %s" % (self.EXECUTABLE, pf, af)

    def pdffonts_command_string(self):
        return "%s %s" % ("pdffonts", self.result().name)

    def process(self):
        env = self.setup_env()

        command = self.preprocess_command_string()
        proc, stdout = self.run_command(command, env)
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)

        command = self.pdffonts_command_string()
        proc, stdout = self.run_command(command, env)
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)

        self.copy_canonical_file()

class HtLatexFilter(SubprocessFilter):
    """
    Generates HTML from LaTeX source.
    """
    INPUT_EXTENSIONS = [".tex", ".txt"]
    OUTPUT_EXTENSIONS = [".html"]
    EXECUTABLES = ['htlatex']
    ALIASES = ['htlatex']
    ADD_NEW_FILES = [".html", ".png", ".css"]

    def command_string(self):
        latexargs = self.args().get('latexargs', '')

        if not '-interaction=' in latexargs:
            latexargs = latexargs + ' -interaction=batchmode'

        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or '',
            'tex4htargs' : self.args().get('tex4htargs', ''),
            't4htargs' : self.args().get('t4htargs', ''),
            'latexargs' : latexargs,
            'script_file' : self.input_filename()
        }
        return """%(prog)s %(script_file)s "%(args)s" "%(tex4htargs)s" "%(t4htargs)s" "%(latexargs)s" """ % args

class AbcFilter(SubprocessFilter):
    """
    Runs abc to convert abc music notation to one of the available output formats.
    """
    ADD_NEW_FILES = False
    ALIASES = ['abc']
    EXECUTABLE = 'abcm2ps'
    FRAGMENT = False
    INPUT_EXTENSIONS = ['.abc']
    OUTPUT_EXTENSIONS = ['.svg', '.html', '.xhtml', '.eps']

    def command_string(self):
        clargs = self.command_line_args() or ''

        if not any(x in clargs for x in ['-E', '-g', '-v', '-X']):
            if self.artifact.ext in ('.eps'):
                output_flag = '-E'
            elif self.artifact.ext in ('.svg'):
                output_flag = '-g'
            elif self.artifact.ext in ('.html', '.xhtml'):
                output_flag = '-X'
            else:
                raise dexy.exceptions.UserFeedback("File extension %s is not supported for abc filter. Supported extensions are %s" % (self.artifact.ext, ", ".join(self.OUTPUT_EXTENSIONS)))
        else:
            output_flag = ''

        args = {
            'prog' : self.executable(),
            'args' : clargs,
            'output_flag' : output_flag,
            'script_file' : self.input_filename(),
            'output_file' : self.output_filename()
        }
        return "%(prog)s %(args)s %(output_flag)s -O %(output_file)s %(script_file)s" % args

    def process(self):
        command = self.command_string()
        proc, stdout = self.run_command(command, self.setup_env())
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)

        if self.artifact.ext in ('.svg', '.eps'):
            # Fix for abcm2ps adding 001 to file name.
            nameparts = os.path.splitext(self.output().name)
            output_filename = "%s001%s" % (nameparts[0], nameparts[1])
            output_filepath = os.path.join(self.artifact.tmp_dir(), output_filename)
            self.output().copy_from_file(output_filepath)
        else:
            self.copy_canonical_file()

        if self.do_add_new_files():
            self.log.debug("adding new files found in %s for %s" % (self.artifact.tmp_dir(), self.artifact.key))
            self.add_new_files()

class AbcMultipleFormatsFilter(SubprocessFilter):
    """
    Runs abc to convert abc music notation to all of the available output formats.
    """
    ALIASES = ['abcm']
    INPUT_EXTENSIONS = ['.abc']
    OUTPUT_EXTENSIONS = ['.json']
    EXECUTABLE = 'abcm2ps'
    ADD_NEW_FILES = False

    def command_string(self, ext):
        clargs = self.command_line_args() or ''

        if any(x in clargs for x in ['-E', '-g', '-v', '-X']):
            raise dexy.exceptions.UserFeedback("Please do not pass any output format flags!")

        if ext in ('.eps'):
            output_flag = '-E'
        elif ext in ('.svg'):
            output_flag = '-g'
        elif ext in ('.html', '.xhtml'):
            output_flag = '-X'
        else:
            raise dexy.exceptions.InternalDexyProblem("bad ext '%s'" % ext)

        args = {
            'prog' : self.executable(),
            'args' : clargs,
            'output_flag' : output_flag,
            'script_file' : self.input_filename(),
            'output_file' : self.output_workfile(ext)
        }
        return "%(prog)s %(args)s %(output_flag)s -O %(output_file)s %(script_file)s" % args

    def output_workfile(self, ext):
        return "%s%s" % (self.artifact.output_data.baserootname(), ext)

    def process(self):
        output = {}

        for ext in ('.eps', '.svg', '.html', '.xhtml'):
            command = self.command_string(ext)
            proc, stdout = self.run_command(command, self.setup_env())
            self.handle_subprocess_proc_return(command, proc.returncode, stdout)

            if ext in ('.svg', '.eps'):
                # Fix for abcm2ps adding 001 to file name.
                nameparts = os.path.splitext(self.output_workfile(ext))
                output_filename = "%s001%s" % (nameparts[0], nameparts[1])
                output_filepath = os.path.join(self.artifact.tmp_dir(), output_filename)
            else:
                output_filename = self.output_workfile(ext)
                output_filepath = os.path.join(self.artifact.tmp_dir(), output_filename)

            with open(output_filepath, "r") as f:
                output[ext] = f.read()

        self.output().set_data(json.dumps(output))

class RdConv(SubprocessFilter):
    """Convert R documentation to other formats."""
    EXECUTABLE = "R CMD Rdconv"
    VERSION_COMMAND = "R CMD Rdconv -v"
    INPUT_EXTENSIONS = ['.Rd']
    OUTPUT_EXTENSIONS = ['.txt', '.html', '.tex', '.R']
    ALIASES = ['rdconv']
    EXTENSION_TO_FORMAT = {
        '.txt' : 'txt',
        '.html' : 'html',
        '.tex' : 'latex',
        '.R' : 'example'
    }

    def command_string(self):
        args = {
                'exe' : self.executable(),
                'clargs' : self.command_line_args() or "",
                'fmt' : self.EXTENSION_TO_FORMAT[self.artifact.ext],
                'input' : self.input_filename(),
                'output' : self.output_filename()
                }
        return "%(exe)s %(clargs)s --type=%(fmt)s --output=%(output)s %(input)s" % args



from dexy.common import OrderedDict
from dexy.filter import DexyFilter
import copy
import dexy.exceptions
import json
import os

class KeyValueStoreFilter(DexyFilter):
    """
    Filter for creating a new key value store on the fly
    """
    ALIASES = ['kv']
    OUTPUT_DATA_TYPE = 'keyvalue'

    def process(self):
        self.output().copy_from_file(self.input().storage.data_file())

        # Call setup() again since it will have created a new blank database.
        self.output().storage.setup()

class HeaderFilter(DexyFilter):
    """
    Apply another file to top of file.
    """
    ALIASES = ['hd']
    KEY_NAME = 'header'

    def find_input_in_parent_dir(self, matches):
        docs = list(self.artifact.doc.node.walk_input_docs())
        docs_d = dict((task.output().long_name(), task) for task in docs)

        if self.artifact.args.has_key(self.KEY_NAME):
            requested = self.artifact.args[self.KEY_NAME]
            if docs_d.has_key(requested):
                matched_key = requested
            else:
                msg = "Couldn't find the %s file %s you requested" % (self.KEY_NAME, requested)
                raise dexy.exceptions.UserFeedback(msg)
        else:
            matched_key = None
            for k in sorted(docs_d.keys()):
                if (os.path.dirname(k) in self.output().parent_dir()) and (matches in k):
                    matched_key = k

        if not matched_key:
            raise dexy.exceptions.UserFeedback("No %s input found for %s" % (self.KEY_NAME, self.artifact.key))

        return docs_d[matched_key].output()

    def process_text(self, input_text):
        header_data = self.find_input_in_parent_dir("_header")
        return "%s\n%s" % (header_data.as_text(), input_text)

class FooterFilter(HeaderFilter):
    """
    Apply another file to bottom of file.
    """
    ALIASES = ['ft']
    KEY_NAME = 'footer'

    def process_text(self, input_text):
        footer_data = self.find_input_in_parent_dir("_footer")
        return "%s\n%s" % (input_text, footer_data.as_text())

class MarkupTagsFilter(DexyFilter):
    """
    Wrap text in specified HTML tags.
    """
    ALIASES = ['tags']

    def process_text(self, input_text):
        tags = copy.copy(self.args()['tags'])
        open_tags = "".join("<%s>" % t for t in tags)
        tags.reverse()
        close_tags = "".join("</%s>" % t for t in tags)

        return "%s\n%s\n%s" % (open_tags, input_text, close_tags)

class StartSpaceFilter(DexyFilter):
    """
    Add a blank space to the start of each line.

    Useful for passing syntax highlighted/preformatted code to mediawiki.
    """
    ALIASES = ['ss', 'startspace']

    @classmethod
    def add_spaces_at_start(self, text, n):
        spaces = " " * n
        return "\n".join("%s%s" % (spaces, line) for line in text.splitlines())

    def process_dict(self, input_dict):
        result = OrderedDict()
        n = self.args().get('n', 1)
        for section_name, section_text in input_dict.iteritems():
            result[section_name] = self.add_spaces_at_start(section_text, n)
        return result

class SectionsByLineFilter(DexyFilter):
    """
    Returns each line in its own section.
    """
    ALIASES = ['lines']
    OUTPUT_DATA_TYPE = 'sectioned'

    def process_text_to_dict(self, input_text):
        data_dict = OrderedDict()
        for i, line in enumerate(input_text.splitlines()):
            data_dict["%s" % (i+1)] = line
        return data_dict

class PrettyPrintJsonFilter(DexyFilter):
    """
    Pretty prints JSON input.
    """
    ALIASES = ['ppjson']
    OUTPUT_EXTENSIONS = ['.json']

    def process_text(self, input_text):
        json_content = json.loads(input_text)
        return json.dumps(json_content, sort_keys=True, indent=4)

class JoinFilter(DexyFilter):
    """
    Takes sectioned code and joins it into a single section. Some filters which
    don't preserve sections will raise an error if they receive multiple
    sections as input, so this forces acknowledgement that sections will be
    lost.
    """
    INPUT_EXTENSIONS = [".*"]
    OUTPUT_EXTENSIONS = [".*"]
    ALIASES = ['join']

    def process(self):
        joined_data = "\n".join(self.artifact.input_data.as_sectioned().values())
        self.artifact.output_data.set_data(joined_data)

class HeadFilter(DexyFilter):
    """
    Returns just the first 10 lines of input.
    """
    ALIASES = ['head']
    def process_text(self, input_text):
        return "\n".join(input_text.split("\n")[0:10]) + "\n"

class WordWrapFilter(DexyFilter):
    """
    Wraps text after 79 characters (tries to preserve existing line breaks and
    spaces).
    """
    ALIASES = ['ww', 'wrap']
    DEFAULT_WIDTH=79

    #http://code.activestate.com/recipes/148061-one-liner-word-wrap-function/
    def wrap_text(self, text, width):
        """
        A word-wrap function that preserves existing line breaks
        and most spaces in the text. Expects that existing line
        breaks are posix newlines (\n).
        """
        return reduce(lambda line, word, width=width: '%s%s%s' %
                 (line,
                   ' \n'[(len(line)-line.rfind('\n')-1
                         + len(word.split('\n',1)[0]
                              ) >= width)],
                   word),
                  text.split(' ')
                 )

    def process_text(self, input_text):
        width = self.args().get('width', self.DEFAULT_WIDTH)
        return self.wrap_text(input_text, width)



from dexy.plugins.process_filters import SubprocessCompileFilter
from dexy.plugins.process_filters import SubprocessCompileInputFilter

class ClangSubprocessCompileFilter(SubprocessCompileFilter):
    """
    Compile code using clang and run.
    """
    ALIASES = ['clang']
    EXECUTABLE = "clang"
    INPUT_EXTENSIONS = [".c"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION = "clang --version"

class ClangSubprocessCompileInputFilter(SubprocessCompileInputFilter):
    """
    Compile code using clang and run with input.
    """
    ALIASES = ['clanginput']
    EXECUTABLE = "clang"
    INPUT_EXTENSIONS = [".c"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION = "clang --version"

class CSubprocessCompileFilter(SubprocessCompileFilter):
    """
    Compile code using gcc and run.
    """
    ALIASES = ['c', 'gcc']
    EXECUTABLE = "gcc"
    INPUT_EXTENSIONS = [".c"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION = "gcc --version"

class CFussySubprocessCompileFilter(SubprocessCompileFilter):
    """
    Compile code using gcc and run, raising an error if compiled code returns nonzero exit.
    """
    ALIASES = ['cfussy']
    EXECUTABLE = "gcc"
    INPUT_EXTENSIONS = [".c"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION = "gcc --version"
    CHECK_RETURN_CODE = True

class CSubprocessCompileInputFilter(SubprocessCompileInputFilter):
    """
    Compile code using gcc and run with input.
    """
    ALIASES = ['cinput']
    EXECUTABLE = "gcc"
    INPUT_EXTENSIONS = [".c"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION = "gcc --version"

class CppSubprocessCompileFilter(SubprocessCompileFilter):
    """
    Compile c++ code using cpp and run.
    """
    ALIASES = ['cpp']
    EXECUTABLE ="c++"
    INPUT_EXTENSIONS = [".cpp"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION = "c++ --version"

class CppSubprocessCompileInputFilter(SubprocessCompileInputFilter):
    """
    Compile c++ code using cpp and run with input.
    """
    ALIASES = ['cppinput']
    EXECUTABLE ="c++"
    INPUT_EXTENSIONS = [".cpp"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION = "c++ --version"



from mock import MagicMock
from datetime import datetime
from dexy.tests.utils import wrap

def test_add_task():
    with wrap() as wrapper:
        wrapper.setup_db()
        attrs = {
                "args" : {},
                "doc.key" : "abc23456",
                'hashstring' : '12345',
                "key_with_batch_id.return_value" : "def1234556",
                "wrapper.batch.batch_id" : 1001,
                "state" : "running",
                "created_by_doc" : None,
                "key" : "file.txt"
                }
        task = MagicMock(**attrs)
        wrapper.db.add_task_before_running(task)
        wrapper.db.conn.commit()

        sql = """select * from tasks"""
        wrapper.db.cursor.execute(sql)
        row = wrapper.db.cursor.fetchone()

        assert row['batch_id'] == 1001
        assert row['key'] == "file.txt"
        assert row['class_name'] == "MagicMock"
        assert row['started_at'] < datetime.now()
        assert not row['created_by_doc']

        assert wrapper.db.next_batch_id() == 1002

def test_update_task():
    with wrap() as wrapper:
        wrapper.setup_db()
        attrs = {
                "args" : {},
                "doc.key" : "abc23456",
                "key_with_batch_id.return_value" : "def1234556",
                "wrapper.batch.batch_id" : 1001,
                "hashstring" : "abc123001",
                "output_data.name" : "abc23456",
                "created_by_doc" : None,
                "output_data_type" : "generic",
                "ext" : ".txt",
                "output_data.storage_type" : "generic",
                "key" : "file.txt"
                }
        task = MagicMock(**attrs)
        wrapper.db.add_task_before_running(task)
        wrapper.db.conn.commit()

        wrapper.db.update_task_after_running(task)

        sql = """select * from tasks"""
        wrapper.db.cursor.execute(sql)
        row = wrapper.db.cursor.fetchone()

        assert row['hashstring'] == 'abc123001'



from dexy.plugins.process_filters import SubprocessInputFilter
import dexy.exceptions

class ApplySed(SubprocessInputFilter):
    """
    A filter which runs on a text file and applies a sed file (specified as an
    input) to that text file. Output is the modified text file.
    """
    ALIASES = ['used']
    EXECUTABLES = ['gsed', 'sed']
    OUTPUT_DATA_TYPE = 'generic'

    def process(self):
        for doc in self.artifact.doc.node.walk_input_docs():
            if doc.output().ext == ".sed":
                command = "%s -f %s" % (self.executable(), doc.output().name)

        if not command:
            raise dexy.exceptions.UserFeedback("A .sed file must be passed as an input to %s" % self.artifact.key)

        proc, stdout = self.run_command(command, self.setup_env(), unicode(self.input()))
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
        self.output().set_data(stdout)

class Sed(SubprocessInputFilter):
    """
    A filter which runs on a sed file and applies this sed file to text files
    passed as inputs. Output is a dict of filenames and output text from each
    input file. If there is only a single input file, output is a dict of
    section names and output text from each section in that input file.
    """
    ALIASES = ['sed']
    EXECUTABLES = ['gsed', 'sed']
    INPUT_EXTENSIONS = [".sed"]
    OUTPUT_EXTENSIONS = [".txt"]
    VERSION_COMMAND = 'sed --version'

    def command_string(self):
        return "%s -f %s" % (self.executable(), self.input_filename())



from datetime import datetime
from dexy.plugin import PluginMeta
import dexy.data
import json

class Database:
    """
    Class that persists run data to a database.
    """
    ALIASES = []
    __metaclass__ = PluginMeta

    @classmethod
    def is_active(klass):
        return True

import sqlite3
class Sqlite3(Database):
    """
    Implementation of dexy database using sqlite3.
    """
    START_BATCH_ID = 1001
    ALIASES = ['sqlite3', 'sqlite']
    FIELDS = [
            ("unique_key", "text"),
            ("batch_id" , "integer"),
            ("key" , "text"),
            ("args" , "text"),
            ("doc_key" , "text"),
            ("canonical_name", "text"),
            ("class_name" , "text"),
            ("hashstring" , "text"),
            ("ext" , "text"),
            ("data_type", "text"),
            ("storage_type", "text"),
            ("created_by_doc" , "text"),
            ("started_at", "timestamp"),
            ("completed_at", "timestamp"),
            ]

    def __init__(self, wrapper):
        self.wrapper = wrapper
        self.conn = sqlite3.connect(
                self.wrapper.db_path(),
                detect_types=sqlite3.PARSE_DECLTYPES
                )
        self.conn.row_factory = sqlite3.Row
        self.cursor = self.conn.cursor()
        self.create_table()
        self._pending_transaction_counter = 0

    def docs(self, n=10):
        """
        Returns the first n tasks of type Doc in the batch.
        """
        sql = "select unique_key, key from tasks where batch_id = ? and class_name = 'Doc' LIMIT ?"
        values = (self.wrapper.batch_id, n,)
        self.cursor.execute(sql, values)
        rows = self.cursor.fetchall()
        return rows

    def query_docs(self, query):
        sql = "select * from tasks where batch_id = ? and key like ? and class_name='Doc'"
        self.cursor.execute(sql, (self.max_batch_id(), "%%%s%%" % query))
        return self.cursor.fetchall()

    def find_filter_artifact_for_doc_key(self, doc_key):
        sql = "select data_type, key, ext, canonical_name, hashstring, args, storage_type from tasks where key=? and batch_id=? AND class_name like '%Artifact'"
        self.cursor.execute(sql, (doc_key, self.wrapper.batch_id,))
        row = self.cursor.fetchone()

        return dexy.data.Data.retrieve(
                row['data_type'],
                row['key'],
                row['ext'],
                row['canonical_name'],
                row['hashstring'],
                json.loads(row['args']),
                self.wrapper,
                row['storage_type'])

    def find_filter_artifact_for_hashstring(self, hashstring):
        sql = "select data_type, key, ext, canonical_name, hashstring, args, storage_type from tasks where hashstring=? and batch_id=? AND class_name like '%Artifact'"
        self.cursor.execute(sql, (hashstring, self.wrapper.batch_id,))
        row = self.cursor.fetchone()

        return dexy.data.Data.retrieve(
                row['data_type'],
                row['key'],
                row['ext'],
                row['canonical_name'],
                row['hashstring'],
                json.loads(row['args']),
                self.wrapper,
                row['storage_type'])

    def find_data_by_websafe_key(self, web_safe_key):
        doc_key = web_safe_key.replace("--", "/")
        return self.find_data_by_doc_key(doc_key)

    def calculate_previous_batch_id(self, current_batch_id):
        sql = "select max(batch_id) as previous_batch_id from tasks where batch_id < ?"
        self.cursor.execute(sql, (current_batch_id,))
        row = self.cursor.fetchone()
        return row['previous_batch_id']

    def get_child_hashes_in_previous_batch(self, parent_hashstring):
        sql = "select * from tasks where batch_id = ? and created_by_doc = ? order by doc_key, started_at"
        self.cursor.execute(sql, (self.wrapper.batch.previous_batch_id, parent_hashstring))
        return self.cursor.fetchall()

    def task_from_previous_batch(self, hashstring):
        sql = "select * from tasks where class_name = 'FilterArtifact' and batch_id = ? and hashstring = ?"
        self.cursor.execute(sql, (self.wrapper.batch.previous_batch_id, hashstring))
        return self.cursor.fetchall()

    def task_from_current_batch(self, hashstring):
        sql = "select * from tasks where class_name = 'FilterArtifact' and batch_id = ? and hashstring = ?"
        self.cursor.execute(sql, (self.wrapper.batch.batch_id, hashstring))
        return self.cursor.fetchall()

    def max_batch_id(self):
        sql = "select max(batch_id) as max_batch_id from tasks"
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        return row['max_batch_id']

    def next_batch_id(self):
        max_batch_id = self.max_batch_id()
        if max_batch_id:
            return max_batch_id + 1
        else:
            return self.START_BATCH_ID

    def serialize_task_args(self, task):
        args_to_serialize = task.args.copy()
        if args_to_serialize.has_key('wrapper'):
            del args_to_serialize['wrapper']
        if args_to_serialize.has_key('inputs'):
            del args_to_serialize['inputs']
        if args_to_serialize.has_key('contents'):
            del args_to_serialize['contents']
            if hasattr(task, 'get_contents_hash'):
                args_to_serialize['contentshash'] = task.get_contents_hash()
                args_to_serialize['data-class-alias'] = task.data_class_alias()

        try:
            serialized_args = json.dumps(args_to_serialize)
        except UnicodeDecodeError:
            msg = "Unable to serialize args. Keys are %s" % args_to_serialize.keys()
            raise dexy.exceptions.InternalDexyProblem(msg)

        return serialized_args

    def add_task_before_running(self, task):
        if hasattr(task, 'doc'):
            doc_key = task.doc.key
        else:
            doc_key = task.key

        attrs = {
                'doc_key' : doc_key,
                'batch_id' : task.wrapper.batch.batch_id,
                'class_name' : task.__class__.__name__,
                'created_by_doc' : task.created_by_doc,
                'hashstring' : task.hashstring,
                'key' : task.key,
                'started_at' : datetime.now(),
                'unique_key' : task.key_with_batch_id()
                }
        self.create_record(attrs)

    def update_task_after_running(self, task):
        if hasattr(task, 'ext'):
            ext = task.ext
            data_type = task.output_data_type
            storage_type = task.output_data.storage_type
            name = task.output_data.name
        else:
            ext = None
            data_type = None
            storage_type = None
            name = None

        attrs = {
                'args' : self.serialize_task_args(task),
                'canonical_name' : name,
                'completed_at' : datetime.now(),
                'ext' : ext,
                'data_type' : data_type,
                'storage_type' : storage_type
                }
        unique_key = task.key_with_batch_id()
        self.update_record(unique_key, attrs)

    def commit(self):
        self.wrapper.log.debug("committing db changes")
        self.conn.commit()

    def save(self):
        self.commit()
        self.conn.close()

    def create_table_sql(self):
        sql = "create table tasks (%s)"
        fields = ["%s %s" % k for k in self.FIELDS]
        return sql % (", ".join(fields))

    def create_table(self):
        try:
            self.conn.execute(self.create_table_sql())
            self.conn.commit()
        except sqlite3.OperationalError as e:
            if e.message != "table tasks already exists":
                raise e

    def create_record(self, attrs):
        keys = sorted(attrs)
        values = [attrs[k] for k in keys]

        qs = ("?," * len(keys))[:-1]
        sql = "insert into tasks (%s) VALUES (%s)" % (",".join(keys), qs)
        self.conn.execute(sql, values)

    def update_record(self, unique_key, attrs):
        keys = sorted(attrs)
        values = [attrs[k] for k in keys]
        updates = ["%s=?" % k for k in keys]

        sql = "update tasks set %s WHERE unique_key=?" % ", ".join(updates)
        values.append(unique_key)

        self.conn.execute(sql, values)

        # make sure we don't go too long before committing changes to sqlite
        self._pending_transaction_counter += 1
        if self._pending_transaction_counter > 200:
            self.commit()
            self._pending_transaction_counter = 0

    def fetch_record(self, unique_key):
        sql = "select * from tasks where unique_key=?"
        self.cursor.execute(sql, (unique_key,))
        rows = self.cursor.fetchall()
        return rows



from StringIO import StringIO
from dexy.common import OrderedDict
from dexy.utils import char_diff
from mock import MagicMock
from nose.exc import SkipTest
import dexy.plugins # make sure plugins are loaded
import dexy.wrapper
import os
import re
import shutil
import sys
import tempfile

TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), 'data')

def create_ordered_dict_from_dict(d):
    od = OrderedDict()
    for k, v in d.iteritems():
        od[k] = v
    return od

class tempdir(object):
    def make_temp_dir(self):
        self.tempdir = tempfile.mkdtemp()
        self.location = os.path.abspath(os.curdir)
        os.chdir(self.tempdir)

    def remove_temp_dir(self):
        os.chdir(self.location)
        try:
            shutil.rmtree(self.tempdir)
        except Exception as e:
            print e
            print "was not able to remove tempdir '%s'" % self.tempdir

    def __enter__(self):
        self.make_temp_dir()

    def __exit__(self, type, value, traceback):
        if not isinstance(value, Exception):
            self.remove_temp_dir()

class wrap(tempdir):
    """
    Create a temporary directory and initialize a dexy wrapper.
    """
    def __enter__(self):
        self.make_temp_dir()
        wrapper = dexy.wrapper.Wrapper()
        wrapper.setup(setup_dirs=True)
        return wrapper

    def __exit__(self, type, value, traceback):
        self.remove_temp_dir()
        if isinstance(value, dexy.exceptions.InactiveFilter):
            raise SkipTest
            return True # swallow InactiveFilter error

class runfilter(wrap):
    """
    Create a temporary directory, initialize a doc and a wrapper, and run the doc.
    """
    def __init__(self, filter_alias, doc_contents, ext=".txt"):
        self.filter_alias = filter_alias
        self.doc_contents = doc_contents
        self.ext = ext

    def __enter__(self):
        self.make_temp_dir()

        doc_key = "subdir/example%s|%s" % (self.ext, self.filter_alias)
        doc_spec = [doc_key, {"contents" : self.doc_contents}]

        try:
            wrapper = dexy.wrapper.Wrapper(doc_spec)
            wrapper.setup(setup_dirs=True)
            wrapper.run()
        except dexy.exceptions.InactiveFilter:
            raise SkipTest

        return wrapper.batch.tree[0].children[0]

def assert_output(filter_alias, doc_contents, expected_output, ext=".txt"):
    if not ext.startswith("."):
        raise Exception("ext arg to assert_in_output must start with dot")

    if isinstance(expected_output, dict):
        expected_output = create_ordered_dict_from_dict(expected_output)
    if isinstance(doc_contents, dict):
        doc_contents = create_ordered_dict_from_dict(doc_contents)

    with runfilter(filter_alias, doc_contents, ext=ext) as doc:
        if expected_output:
            try:
                assert doc.output().data() == expected_output
            except AssertionError as e:
                if not isinstance(expected_output, OrderedDict):
                    print char_diff(unicode(doc.output()), expected_output)
                else:
                    print "Output: %s" % doc.output().data()
                    print "Expected: %s" % expected_output

                raise e
        else:
            raise Exception("Output is '%s'" % doc.output().data())

def assert_output_matches(filter_alias, doc_contents, expected_regex, ext=".txt"):
    if not ext.startswith("."):
        raise Exception("ext arg to assert_in_output must start with dot")

    with runfilter(filter_alias, doc_contents, ext=ext) as doc:
        if expected_regex:
            assert re.match(expected_regex, unicode(doc.output()))
        else:
            raise Exception(unicode(doc.output()))

def assert_output_cached(filter_alias, doc_contents, ext=".txt", min_filesize=None):
    if not ext.startswith("."):
        raise Exception("ext arg to assert_output_cached must start with dot")

    with runfilter(filter_alias, doc_contents, ext=ext) as doc:
        assert doc.output().is_cached()
        if min_filesize:
            assert doc.output().filesize() > min_filesize

def assert_in_output(filter_alias, doc_contents, expected_output, ext=".txt"):
    if not ext.startswith("."):
        raise Exception("ext arg to assert_in_output must start with dot")

    with runfilter(filter_alias, doc_contents, ext=ext) as doc:
        if expected_output:
            assert expected_output in unicode(doc.output())
        else:
            raise Exception(unicode(doc.output()))

class divert_stdout():
    def __enter__(self):
        self.old_stdout = sys.stdout
        self.my_stdout = StringIO()
        sys.stdout = self.my_stdout
        return self.my_stdout

    def __exit__(self, type, value, traceback):
        sys.stdout = self.old_stdout
        self.my_stdout.close()

class divert_stderr():
    def __enter__(self):
        self.old_stderr = sys.stderr
        self.my_stderr = StringIO()
        sys.stderr = self.my_stderr
        return self.my_stderr

    def __exit__(self, type, value, traceback):
        sys.stderr = self.old_stderr
        self.my_stderr.close()

class run_templating_plugin():
    def __init__(self, klass, mock_attrs=None):
        if not mock_attrs:
            mock_attrs = {}
        self.f = MagicMock(**mock_attrs)
        self.plugin = klass(self.f)

    def __enter__(self):
        env = self.plugin.run()
        assert isinstance(env, dict)
        return env

    def __exit__(self, type, value, traceback):
        pass



from dexy.tests.utils import wrap
from dexy.doc import Doc

def test_pydoc_filter():
    with wrap() as wrapper:
        doc = Doc("modules.txt|pydoc", contents="os math", wrapper=wrapper)
        wrapper.run_docs(doc)



from dexy.common import OrderedDict
from dexy.plugins.pygments_filters import PygmentsFilter
from idiopidae.runtime import Composer
from idiopidae.parser import IdiopidaeParser, IdiopidaeParserScanner
from pygments.formatters import get_all_formatters
import dexy.exceptions
import idiopidae.parser
import json
import re
import zapps.rt

class IdioFilter(PygmentsFilter):
    """
    Apply idiopidae to split document into sections at ### @export
    "section-name" comments.
    """
    ALIASES = ['idio', 'idiopidae']
    OUTPUT_EXTENSIONS = PygmentsFilter.MARKUP_OUTPUT_EXTENSIONS + PygmentsFilter.IMAGE_OUTPUT_EXTENSIONS + [".txt"]

    @classmethod
    def data_class_alias(klass, file_ext):
        if file_ext in PygmentsFilter.MARKUP_OUTPUT_EXTENSIONS + [".txt"]:
            return 'sectioned'
        else:
            return 'generic'

    def do_add_new_files(self):
        if self.args().get('add-new-files', False) or self.args().get("add_new_files", False):
            return True
        elif self.artifact.ext in self.IMAGE_OUTPUT_EXTENSIONS:
            return True
        else:
            return False

    def process(self):
        args = self.args().copy()
        lexer = self.create_lexer_instance(args)

        input_text = self.input().as_text()
        composer = Composer()

        try:
            P = IdiopidaeParser(IdiopidaeParserScanner(input_text + "\n\0"))
            builder = P.Document()
        except zapps.rt.SyntaxError as s:
            zapps_err = zapps.rt.print_error(input_text + "\n\0", s, P._scanner, False)
            msg = "Idiopidae was unable to parse input for %s\n%s" % (self.artifact.key, zapps_err)
            raise dexy.exceptions.UserFeedback(msg)
        except zapps.rt.NoMoreTokens as s:
            msg = "Could not complete parsing; stopped around here:%s" % P._scanner
            raise dexy.exceptions.UserFeedback(msg)

        output_dict = OrderedDict()
        all_lines = []
        lineno = 1

        add_new_files = self.do_add_new_files()

        for i, s in enumerate(builder.sections):
            self.log.debug("In section no. %s name %s" % (i, s))
            lines = builder.statements[i]['lines']
            all_lines.extend(lines)

            if len(lines) == 0:
                next
            if not re.match("^\d+$", s):
                # Manually named section, the sectioning comment takes up a
                # line, so account for this to keep line nos in sync.
                lineno += 1

            formatter = self.create_formatter_instance(args)

            if hasattr(formatter, 'linenostart'):
                formatter.linenostart = lineno
            elif hasattr(formatter, 'line_number_start'):
                formatter.line_number_start = lineno

            formatted_lines = composer.format(lines, lexer, formatter)

            if add_new_files:
                new_doc_name = "%s--%s%s" % (self.artifact.doc.key.replace("|", "--"), s, self.artifact.ext)
                doc = self.add_doc(new_doc_name, formatted_lines)

            if not self.artifact.ext in self.IMAGE_OUTPUT_EXTENSIONS:
                if add_new_files:
                    doc.canon = False
                output_dict[s] = formatted_lines

            lineno += len(lines)

        if self.artifact.ext in self.IMAGE_OUTPUT_EXTENSIONS:
            formatter = self.create_formatter_instance(args)
            formatted_lines = composer.format(lines, lexer, formatter)
            self.output().set_data(formatted_lines)
        else:
            self.output().set_data(output_dict)

class IdioMultipleFormatsFilter(PygmentsFilter):
    """
    Apply idiopidae to split document into sections at ### @export
    "section-name" comments, then apply syntax highlighting for all available
    text-based formats.
    """
    ALIASES = ['idiom']
    OUTPUT_EXTENSIONS = ['.json']

    def create_formatter_instance(self, args, formatter_class):
        formatter_args = {'lineanchors' : self.output().web_safe_document_key() }

        # Python 2.6 doesn't like unicode keys as kwargs
        for k, v in args.iteritems():
            formatter_args[str(k)] = v

        self.log.debug("creating formatter of type %s with args %s" % (formatter_class.__name__, args))
        return formatter_class(**formatter_args)

    def process(self):
        input_text = self.input().as_text()
        composer = Composer()
        builder = idiopidae.parser.parse('Document', input_text + "\n\0")

        args = self.args().copy()
        lexer = self.create_lexer_instance(args)

        formatters = []
        for formatter_class in get_all_formatters():
            formatters.append(self.create_formatter_instance(args, formatter_class))

        output_dict = OrderedDict()
        lineno = 1

        for i, s in enumerate(builder.sections):
            self.log.debug("In section no. %s name %s" % (i, s))
            lines = builder.statements[i]['lines']
            if len(lines) == 0:
                next
            if not re.match("^\d+$", s):
                # Manually named section, the sectioning comment takes up a
                # line, so account for this to keep line nos in sync.
                lineno += 1

            output_dict[s] = {}
            for formatter in formatters:
                formatter.linenostart = lineno
                formatted_lines = composer.format(lines, lexer, formatter)

                for filename in formatter.filenames:
                    ext = filename.lstrip("*")
                    if not ext in self.IMAGE_OUTPUT_EXTENSIONS:
                        output_dict[s][ext] = formatted_lines

            lineno += len(lines)

        self.output().set_data(json.dumps(output_dict))



import inspect
import dexy.exceptions

class PluginMeta(type):
    """
    Based on http://martyalchin.com/2008/jan/10/simple-plugin-framework/
    """
    def __init__(cls, name, bases, attrs):
        if not hasattr(cls, 'plugins'):
            # This branch only executes when processing the mount point itself.
            # So, since this is a new plugin type, not an implementation, this
            # class shouldn't be registered as a plugin. Instead, it sets up a
            # list where plugins can be registered later.
            cls.plugins = []
            cls.aliases = {}
            cls.source = {}
            cls.source[cls.__name__] = inspect.getsource(cls)
        else:
            # This must be a plugin implementation, which should be registered.
            # Simply appending it to the list is all that's needed to keep
            # track of it later.
            cls.plugins.append(cls)
            cls.source[cls.__name__] = inspect.getsource(cls)

            if not inspect.getdoc(cls):
                breadcrumbs = " -> ".join(t.__name__ for t in inspect.getmro(cls)[:-1][::-1])
                raise dexy.exceptions.InternalDexyProblem("no docstring found for dexy plugin '%s' (%s, defined in %s), docstrings are required" % (cls.__name__, breadcrumbs, cls.__module__))

            basenames = [k.__name__ for k in cls.__bases__]

            if hasattr(cls, 'ALIASES'):
                for alias in cls.ALIASES:

                    # Namespace templates by their plugin name (after dexy_)
                    if 'Template' in basenames:
                        if cls.__module__ == 'dexy.plugins.templates':
                            prefix = 'dexy'
                        else:
                            prefix = cls.__module__.replace("dexy_", "")
                        alias = "%s:%s" % (prefix, alias)

                    if alias in cls.aliases:
                        raise Exception("duplicate alias %s found in %s, already present in %s" % (alias, cls.__name__, cls.aliases[alias].__name__))
                    cls.aliases[alias] = cls
            elif hasattr(cls, 'NAMESPACE'):
                cls.aliases[cls.NAMESPACE] = cls

class Command:
    NAMESPACE = None
    DEFAULT_COMMAND = None
    __metaclass__ = PluginMeta

    @classmethod
    def is_active(klass):
        return True



from dexy.plugins.templates import DefaultTemplate
from dexy.tests.utils import tempdir
import os

def test_run():
    with tempdir():
        DefaultTemplate.run("test")
        assert not os.path.exists("dexy.rst")

def test_dexy():
    for batch in DefaultTemplate.dexy():
        assert 'jinja' in batch.filters_used()
        assert "Doc:hello.txt|jinja" in batch.lookup_table
        assert "Doc:dexy.rst|jinja|rst2html" in batch.lookup_table

def test_validate_default():
    DefaultTemplate.validate()



from dexy.filter import DexyFilter
from dexy.plugins.templating_plugins import TemplatePlugin
from jinja2 import FileSystemLoader
from jinja2.exceptions import TemplateSyntaxError
from jinja2.exceptions import UndefinedError
from jinja2.exceptions import TemplateNotFound
import dexy.exceptions
import jinja2
import os
import re
import traceback

class TemplateFilter(DexyFilter):
    """
    Base class for templating system filters such as JinjaFilter. Templating
    systems are used to make generated artifacts available within documents.

    Plugins are used to prepare content.
    """
    ALIASES = ['template']
    FRAGMENT = False

    def template_plugins(self):
        """
        Returns a list of plugin classes for run_plugins to use.
        """
        if self.args().get('plugins'):
            return [TemplatePlugin.aliases[alias] for alias in self.args()['plugins']]
        else:
            return TemplatePlugin.plugins

    def run_plugins(self):
        env = {}
        for plugin_class in self.template_plugins():
            self.log.debug("Running template plugin %s" % plugin_class.__name__)
            plugin = plugin_class(self)
            new_env_vars = plugin.run()
            if any(v in env.keys() for v in new_env_vars):
                new_keys = ", ".join(sorted(new_env_vars))
                existing_keys = ", ".join(sorted(env))
                msg = "plugin class '%s' is trying to add new keys '%s', already have '%s'"
                raise dexy.exceptions.InternalDexyProblem(msg % (plugin_class.__name__, new_keys, existing_keys))
            env.update(new_env_vars)
        return env

    def process_text(self, input_text):
        template_data = self.run_plugins()
        return input_text % template_data

class JinjaFilter(TemplateFilter):
    """
    Runs the Jinja templating engine.
    """
    ALIASES = ['jinja']
    FRAGMENT = False

    def setup_jinja_env(self, loader=None):
        env_attrs = self.args().copy()

        # Remove jinja attrs not intended for env
        if env_attrs.has_key('vars'):
            del env_attrs['vars']
        if env_attrs.has_key('variables'):
            del env_attrs['variables']
        if env_attrs.has_key('ext'):
            del env_attrs['ext']
        if env_attrs.has_key('changetags'):
            changetags = env_attrs['changetags']
            del env_attrs['changetags']
        else:
            changetags = True

        env_attrs['undefined'] = jinja2.StrictUndefined

        if self.artifact.ext in (".tex", ".wiki") and changetags:
            env_attrs.setdefault('block_start_string', '<%')
            env_attrs.setdefault('block_end_string', '%>')
            env_attrs.setdefault('variable_start_string', '<<')
            env_attrs.setdefault('variable_end_string', '>>')
            env_attrs.setdefault('comment_start_string', '<#')
            env_attrs.setdefault('comment_end_string', '#>')

        if loader:
            env_attrs['loader'] = loader

        debug_attr_string = ", ".join("%s: %r" % (k, v) for k, v in env_attrs.iteritems())
        self.log.debug("creating jinja2 environment with: %s" % debug_attr_string)
        return jinja2.Environment(**env_attrs)

    def handle_jinja_exception(self, e, input_text, template_data):
        result = []
        input_lines = input_text.splitlines()

        # Try to parse line number from stack trace...
        if isinstance(e, UndefinedError) or isinstance(e, TypeError):
            # try to get the line number
            m = re.search(r"File \"<template>\", line ([0-9]+), in top\-level template code", traceback.format_exc())
            if m:
                e.lineno = int(m.groups()[0])
            else:
                e.lineno = 0
                self.log.warn("unable to parse line number from traceback")

        args = {
                'error_type' : e.__class__.__name__,
                'key' : self.artifact.key,
                'lineno' : e.lineno,
                'message' : e.message,
                'name' : self.output().name,
                'workfile' : self.input().storage.data_file()
                }

        result.append("a %(error_type)s problem was detected: %(message)s" % args)

        if isinstance(e, UndefinedError):
            match_has_no_attribute = re.match("^'[\w\s\.]+' has no attribute '(.+)'$", e.message)
            match_is_undefined = re.match("^'([\w\s]+)' is undefined$", e.message)

            if match_has_no_attribute:
                undefined_object = match_has_no_attribute.groups()[0]
                match_lines = []
                for i, line in enumerate(input_lines):
                    if (".%s" % undefined_object in line) or ("'%s'" % undefined_object in line) or ("\"%s\"" % undefined_object in line):
                        result.append("line %04d: %s" % (i+1, line))
                        match_lines.append(i)
                if len(match_lines) == 0:
                    raise dexy.exceptions.InternalDexyProblem("Tried to find source of: %s. Could not find match for '%s'" % (e.message, undefined_object))

            elif match_is_undefined:
                undefined_object = match_is_undefined.groups()[0]
                for i, line in enumerate(input_lines):
                    if undefined_object in line:
                        result.append("line %04d: %s" % (i+1, line))
            else:
                raise dexy.exceptions.InternalDexyProblem("don't know how to match pattern: %s" % e.message)
        else:
            result.append("line %04d: %s" % (e.lineno, input_lines[e.lineno-1]))

        raise dexy.exceptions.UserFeedback("\n".join(result))

    def process(self):
        wd = self.setup_wd()
        dirs = ['.', wd, self.artifact.tmp_dir()]
        self.log.debug("setting up jinja FileSystemLoader with dirs %s" % ", ".join(dirs))
        loader = FileSystemLoader(dirs)

        self.log.debug("setting up jinja environment")
        env = self.setup_jinja_env(loader=loader)

        env.filters['pygmentize'] = dexy.plugins.templating_plugins.PygmentsStylesheet.highlight
        env.filters['rstcode'] = dexy.plugins.templating_plugins.RstCode.rstcode
        env.filters['indent'] = dexy.plugins.templating_plugins.JinjaFilters.indent
        env.filters['head'] = dexy.plugins.templating_plugins.JinjaFilters.head
        env.filters['javadoc2rst'] = dexy.plugins.templating_plugins.JavadocToRst.javadoc2rst
        if dexy.plugins.templating_plugins.PrettyPrintHtml.is_active():
            env.filters['prettify_html'] = dexy.plugins.templating_plugins.PrettyPrintHtml.prettify_html

        self.log.debug("initializing template")

        template_data = self.run_plugins()
        self.log.debug("jinja template data keys are %s" % ", ".join(sorted(template_data)))

        try:
            self.log.debug("about to create jinja template")
            template = env.get_template(self.input_filename())
            self.log.debug("about to process jinja template")
            template.stream(template_data).dump(self.output_filepath(), encoding="utf-8")
        except (TemplateSyntaxError, UndefinedError, TypeError) as e:
            if os.path.exists(self.output_filepath()):
                self.log.debug("removing %s since jinja had an error" % self.output_filepath())
                os.remove(self.output_filepath())
            self.handle_jinja_exception(e, self.input().as_text(), template_data)
        except TemplateNotFound as e:
            raise dexy.exceptions.UserFeedback("Jinja couldn't find the template '%s', make sure this file is an input to %s" % (e.message, self.artifact.doc.key))
        except Exception as e:
            if os.path.exists(self.output_filepath()):
                self.log.debug("removing %s since jinja had an error" % self.output_filepath())
                os.remove(self.output_filepath())
            self.log.debug(str(e))
            raise



import dexy.plugins.api_filters
import dexy.plugins.archive_filters
import dexy.plugins.boto_filters
import dexy.plugins.clang_filters
import dexy.plugins.deprecated_filters
import dexy.plugins.example_filters
import dexy.plugins.file_extension_filters
import dexy.plugins.idio_filters
import dexy.plugins.java_filters
import dexy.plugins.latex_filters
import dexy.plugins.markdown_filters
import dexy.plugins.nltk_filter
import dexy.plugins.output_reporters
import dexy.plugins.parsers
import dexy.plugins.pexpect_filters
import dexy.plugins.phantomjs_filters
import dexy.plugins.pydoc_filters
import dexy.plugins.pygments_filters
import dexy.plugins.pygments_plugins
import dexy.plugins.restructured_text_filters
import dexy.plugins.run_reporter
import dexy.plugins.split_filters
import dexy.plugins.standard_filters
import dexy.plugins.stationery
import dexy.plugins.stdout_filters
import dexy.plugins.stdout_input_filters
import dexy.plugins.subprocess_filters
import dexy.plugins.templates
import dexy.plugins.templating_filters
import dexy.plugins.website_reporters
import dexy.plugins.wordpress_filters
import dexy.plugins.yamlargs_filters

import pkg_resources
# Automatically register plugins in any python package named like dexy_*
for dist in pkg_resources.working_set:
    if dist.key.startswith("dexy-"):
        import_pkg = dist.egg_name().split("-")[0]
        __import__(import_pkg)



from dexy.commands import init_wrapper
from dexy.doc import Doc
from dexy.exceptions import UserFeedback
from dexy.tests.utils import tempdir
from dexy.tests.utils import wrap
from dexy.wrapper import Wrapper
from nose.tools import raises
import os

def test_config_for_directory():
    with wrap() as wrapper:
        with open("docs.yaml", "w") as f:
            f.write(""".abc""")

        with open("root.abc", "w") as f:
            f.write("hello")

        with open("root.def", "w") as f:
            f.write("hello")

        os.makedirs("s1")
        os.makedirs("s2")

        with open("s1/s1.abc", "w") as f:
            f.write("hello")

        with open("s1/s1.def", "w") as f:
            f.write("hello")

        with open("s2/s2.abc", "w") as f:
            f.write("hello")

        with open("s2/s2.def", "w") as f:
            f.write("hello")

        with open(os.path.join('s1', 'docs.yaml'), 'w') as f:
            f.write(""".def|dexy""")

        wrapper.setup_config()
        wrapper.run()

        assert len(wrapper.batch.tasks()) == 8

        p = wrapper.batch.task("PatternNode:*.abc")
        c = wrapper.batch.task("Doc:s2/s2.abc")
        assert c in p.children

def test_config_file():
    with tempdir():
        with open("dexy.conf", "w") as f:
            f.write("""{ "logfile" : "a.log" }""")

        wrapper = init_wrapper({'conf' : 'dexy.conf'})
        assert wrapper.log_file == "a.log"

def test_kwargs_override_config_file():
    with tempdir():
        with open("dexy.conf", "w") as f:
            f.write("""{ "logfile" : "a.log" }""")

        wrapper = init_wrapper({
            '__cli_options' : { 'logfile' : 'b.log' },
            'logfile' : "b.log",
            'conf' : 'dexy.conf'
            })
        assert wrapper.log_file == "b.log"

def test_wrapper_init():
    wrapper = Wrapper()
    assert wrapper.artifacts_dir == 'artifacts'

def test_wrapper_setup():
    with tempdir():
        assert not os.path.exists('artifacts')
        wrapper = init_wrapper({'conf' : 'dexy.conf'})
        wrapper.setup_dexy_dirs()
        assert os.path.exists('artifacts')

def test_wrapper_run():
    with tempdir():
        wrapper = init_wrapper({'conf' : 'dexy.conf'})
        wrapper.setup_dexy_dirs()
        d1 = Doc("abc.txt|outputabc", contents="these are the contents", wrapper=wrapper)
        d2 = Doc("hello.txt|outputabc", contents="these are more contents", wrapper=wrapper)
        assert d1.state == 'new'
        assert d2.state == 'new'
        wrapper.run_docs(d1, d2)
        assert d1.state == 'complete'
        assert d2.state == 'complete'

def test_wrapper_register():
    with tempdir():
        doc = Doc("abc.txt")
        wrapper = init_wrapper({'conf' : 'dexy.conf'})
        wrapper.setup_dexy_dirs()
        wrapper.setup_log()
        wrapper.setup_db()
        wrapper.setup_batch()
        wrapper.batch.add_doc(doc)
        assert doc in wrapper.batch.tasks()

@raises(UserFeedback)
def test_double_config_raises_error():
    with wrap() as wrapper:
        with open("dexy.txt", "w") as f:
            f.write("hello.txt")
        with open("dexy.yaml", "w") as f:
            f.write("hello.txt")

        wrapper.load_doc_config()

def test_no_config_raises_error():
    with wrap() as wrapper:
        wrapper.load_doc_config()



from dexy.reporter import Reporter
from jinja2 import Environment
from jinja2 import FileSystemLoader
import datetime
import operator
import os
import random
import shutil

class RunReporter(Reporter):
    """
    Returns info about a dexy run.
    """
    ALLREPORTS = True
    ALIASES = ['run']

    def run(self, wrapper):
        latest_report_dir = os.path.join(wrapper.log_dir, 'run-latest')

        timestamp = datetime.datetime.now().strftime("%Y-%m-%d--%H-%M-%S")
        report_dir = os.path.join(wrapper.log_dir, "run-%s" % timestamp)
        report_filename = os.path.join(report_dir, 'index.html')

        # Remove any existing directory (unlikely)
        shutil.rmtree(report_dir, ignore_errors=True)

        # Copy template files (e.g. .css files)
        template_dir = os.path.join(os.path.dirname(__file__), 'files')
        shutil.copytree(template_dir, report_dir)

        env_data = {}

        env_data['attrgetter'] = operator.attrgetter
        env_data['dict'] = dict
        env_data['float'] = float
        env_data['hasattr'] = hasattr
        env_data['isinstance'] = isinstance
        env_data['len'] = len
        env_data['sorted'] = sorted

        env_data['batch'] = wrapper.batch

        env_data['docs'] = wrapper.batch.docs()
        env_data['wrapper'] = wrapper

        def printable_args(args):
            return dict((k, v) for k, v in args.iteritems() if not k in ('contents', 'wrapper'))

        def print_children(node, indent=0):
            rand_id = random.randint(10000000,99999999)
            spaces = " " * 4 * indent
            nbspaces = "&nbsp;" * 4 * indent
            content = []
            if node.__class__.__name__ == 'Doc':
                link_if_doc = """&nbsp;<a href="#%s">&darr; doc info</a>""" % node.websafe_key()
            else:
                link_if_doc = ""
            content.append("""%s<div data-toggle="collapse" data-target="#%s">%s%s%s</div>""" % (spaces, rand_id, nbspaces, node.key_with_class(), link_if_doc))
            content.append("""  %s<div id="%s" class="collapse">""" % (spaces, rand_id))
            for child in node.children:
                if not "Artifact" in child.__class__.__name__:
                    for line in print_children(child, indent+1):
                        content.append(line)
            content.append("  %s</div>" % spaces)
            return content 

        env_data['print_children'] = print_children
        env_data['printable_args'] = printable_args

        env = Environment()
        env.loader = FileSystemLoader(os.path.dirname(__file__))
        template = env.get_template('template.html')

        template.stream(env_data).dump(report_filename, encoding="utf-8")

        # Copy this to run-latest
        # TODO symlink instead?
        shutil.rmtree(latest_report_dir, ignore_errors=True)
        shutil.copytree(report_dir, latest_report_dir)



import dexy.doc
import dexy.task
import fnmatch
import os
import re

class Node(dexy.task.Task):
    """
    base class for Nodes
    """
    ALIASES = ['node']

    def __init__(self, key, **kwargs):
        super(Node, self).__init__(key, **kwargs)
        self.inputs = list(kwargs.get('inputs', []))

    def hashstring(self):
        return self.metadata.compute_hash()

    def walk_inputs(self):
        """
        Returns a generator which recursively yields all inputs and their inputs.
        """
        def walk(node, level=0):
            for inpt in node.inputs:
                for t in walk(inpt,level+1):
                    yield t
            if level > 0:
                yield node

        return walk(self)        

    def walk_input_docs(self):
        for node in self.walk_inputs():
            for child in node.children:
                yield child

    def setup(self):
        self.metadata.input_hashstrings = ",".join(i.hashstring for i in self.inputs)
        self.set_hashstring()

        # Now update child hashstrings for inputs.
        for doc in self.children:
            for artifact in doc.children[1:]:
                artifact.metadata.node_hashstring = self.hashstring
                artifact.set_hashstring()
                artifact.setup_output_data()
                artifact.input_data = artifact.prior.output_data

        # Now update node's hashstring for children, won't affect children but
        # will affect other docs using this node as an input.
        self.metadata.child_hashstrings = ",".join(c.hashstring for c in self.children)
        self.set_hashstring()

class DocNode(Node):
    """
    Node representing a single doc.
    """
    ALIASES = ['doc']

    def populate(self):
        doc = dexy.doc.Doc(self.key, **self.args)
        if not hasattr(doc, 'wrapper') or not doc.wrapper:
            doc.wrapper = self.wrapper
        doc.node = self
        self.children.append(doc)
        doc.populate()
        doc.transition('populated')

class BundleNode(Node):
    """
    Node representing a bundle of other nodes.
    """
    ALIASES = ['bundle']

    def populate(self):
        pass

class ScriptNode(BundleNode):
    """
    Node representing a bundle of other nodes which must always run in a set
    order, so if any of the bundle siblings change, the whole bundle should be
    re-run.
    """
    ALIASES = ['script']

    def setup(self):
        self.metadata.input_hashstrings = ",".join(i.hashstring for i in self.inputs)
        self.metadata.child_hashstrings = ",".join(c.hashstring for c in self.children)
        self.set_hashstring()

        for doc in self.children:
            for artifact in doc.children[1:]:
                artifact.metadata.node_hashstring = self.hashstring
                artifact.set_hashstring()

class PatternNode(Node):
    """
    A node which takes a file matching pattern and creates individual Doc
    objects for all files that match the pattern.
    """
    ALIASES = ['pattern']

    def populate(self):
        self.set_log()

        file_pattern = self.key.split("|")[0]
        filter_aliases = self.key.split("|")[1:]

        recurse = self.args.get('recurse', True)
        for dirpath, filename in self.wrapper.walk(".", recurse):
            raw_filepath = os.path.join(dirpath, filename)
            filepath = os.path.normpath(raw_filepath)

            if fnmatch.fnmatch(filepath, file_pattern):
                except_p = self.args.get('except')
                if except_p and re.search(except_p, filepath):
                    self.log.debug("skipping file '%s' because it matches except '%s'" % (filepath, except_p))
                else:
                    if len(filter_aliases) > 0:
                        doc_key = "%s|%s" % (filepath, "|".join(filter_aliases))
                    else:
                        doc_key = filepath

                    if hasattr(self.wrapper.batch, 'ast'):
                        doc_args = self.wrapper.batch.ast.default_args_for_directory(filepath)
                    else:
                        doc_args = {}

                    doc_args.update(self.args_before_defaults)
                    doc_args['wrapper'] = self.wrapper

                    # TODO implement 'depends'
                    #if doc_args.has_key('depends'):
                    #    if doc_args.get('depends'):
                    #        doc_children = self.wrapper.registered_docs()
                    #    else:
                    #        doc_children = []
                    #    del doc_args['depends']

                    self.log.debug("creating child of patterndoc %s: %s" % (self.key, doc_key))
                    self.log.debug("with args %s" % doc_args)
                    doc = dexy.doc.Doc(doc_key, **doc_args)
                    doc.node = self
                    self.children.append(doc)
                    doc.populate()
                    doc.transition('populated')




from dexy.filter import DexyFilter
import inspect
import json
import pkgutil
import sys

class PythonDocumentationFilter(DexyFilter):
    """
    Returns doc info for named python modules.
    """
    ALIASES = ["pydoc"]
    INPUT_EXTENSIONS = [".txt"]
    OUTPUT_DATA_TYPE = 'keyvalue'
    OUTPUT_EXTENSIONS = ['.sqlite3', '.json']

    def fetch_item_content(self, key, item):
        is_method = inspect.ismethod(item)
        is_function = inspect.isfunction(item)
        if is_method or is_function:
            # Get source code
            try:
                source = inspect.getsource(item)
            except IOError:
                source = ""

            self.add_source_for_key(key, source)

            self.output().append("%s:doc" % key, inspect.getdoc(item))
            self.output().append("%s:comments" % key, inspect.getcomments(item))

        else: # not a function or a method
            try:
                # If this can be JSON-serialized, leave it alone...
                json.dumps(item)
                self.add_source_for_key(key, item)
            except TypeError:
                # ... if it can't, convert it to a string to avoid problems.
                self.add_source_for_key(key, str(item))

    def add_source_for_key(self, key, source):
        """
        Appends source code + syntax highlighted source code to persistent store.
        """
        self.output().append("%s:value" % key, source)
        if not (type(source) == str or type(source) == unicode):
            source = inspect.getsource(source)
        self.output().append("%s:source" % key, source)

    def process_members(self, package_name, mod):
        """
        Process all members of the package or module passed.
        """
        name = mod.__name__

        for k, m in inspect.getmembers(mod):
            self.log.debug("in %s processing element %s" % (mod.__name__, k))
            if not inspect.isclass(m) and hasattr(m, '__module__') and m.__module__ and m.__module__.startswith(package_name):
                key = "%s.%s" % (m.__module__, k)
                self.fetch_item_content(key, m)

            elif inspect.isclass(m) and m.__module__.startswith(package_name):
                key = "%s.%s" % (mod.__name__, k)
                try:
                    item_content = inspect.getsource(m)
                    self.output().append("%s:doc" % key, inspect.getdoc(m))
                    self.output().append("%s:comments" % key, inspect.getcomments(m))
                    self.add_source_for_key(key, item_content)
                except IOError:
                    self.log.debug("can't get source for %s" % key)
                    self.add_source_for_key(key, "")

                try:
                    for ck, cm in inspect.getmembers(m):
                        key = "%s.%s.%s" % (name, k, ck)
                        self.fetch_item_content(key, cm)
                except AttributeError:
                    pass

            else:
                key = "%s.%s" % (name, k)
                self.fetch_item_content(key, m)

    def process_module(self, package_name, name):
        try:
            self.log.debug("Trying to import %s" % name)
            __import__(name)
            mod = sys.modules[name]
            self.log.debug("Success importing %s" % name)

            try:
                module_source = inspect.getsource(mod)
                json.dumps(module_source)
                self.add_source_for_key(name, inspect.getsource(mod))
            except (UnicodeDecodeError, IOError, TypeError):
                self.log.debug("Unable to load module source for %s" % name)

            self.process_members(package_name, mod)

        except (ImportError, TypeError) as e:
            self.log.debug(e)

    def process(self):
        """
        input_text should be a list of installed python libraries to document.
        """
        package_names = self.input().as_text().split()
        packages = [__import__(package_name) for package_name in package_names]

        for package in packages:
            self.log.debug("processing package %s" % package)
            package_name = package.__name__
            prefix = package.__name__ + "."

            self.process_members(package_name, package)

            if hasattr(package, '__path__'):
                for module_loader, name, ispkg in pkgutil.walk_packages(package.__path__, prefix=prefix):
                    self.log.debug("in package %s processing module %s" % (package_name, name))
                    if not name.endswith("__main__"):
                        self.process_module(package_name, name)
            else:
                self.process_module(package.__name__, package.__name__)

        self.output().save()



import dexy.plugin
import os
import sys
import shutil
import dexy.tests.utils
import dexy.wrapper

class Template(object):
    ALIASES = []
    FILTERS_USED = []
    __metaclass__ = dexy.plugin.PluginMeta

    @classmethod
    def is_active(klass):
        return True

    @classmethod
    def template_source_dir(klass):
        template_install_dir = os.path.dirname(sys.modules[klass.__module__].__file__)

        if hasattr(klass, 'CONTENTS'):
            contents_dirname = klass.CONTENTS
        else:
            # default is to have contents in directory with same name as alias followed by "-template"
            contents_dirname = "%s-template" % klass.ALIASES[0]

        return os.path.join(template_install_dir, contents_dirname)

    @classmethod
    def run(klass, directory, **kwargs):
        if os.path.exists(directory):
            raise dexy.exceptions.UserFeedback("Directory '%s' already exists, aborting." % directory)
        source = klass.template_source_dir()
        shutil.copytree(source, directory)

        dexy_rst = os.path.join(directory, 'dexy.rst')
        if os.path.exists(dexy_rst):
            if not kwargs.get('meta'):
                os.remove(dexy_rst)

    @classmethod
    def dexy(klass, meta=True, additional_doc_keys=None):
        """
        Run dexy on this template's files in a temporary directory.

        Yields the batch object for the dexy run, so we can call methods on it
        while still in the tempdir.
        """
        DOC_KEYS = [
                "dexy.yaml|idio|t",
                "dexy.rst|idio|t",
                "dexy.rst|jinja|rst2html",
                "dexy.rst|jinja|rst2man"
                ]

        with dexy.tests.utils.tempdir():
            # Copy files to directory 'ex'
            klass.run("ex", meta=meta)

            # Run dexy in directory 'ex'
            os.chdir("ex")
            wrapper = dexy.wrapper.Wrapper()
            wrapper.setup(True)
            wrapper.batch = dexy.batch.Batch(wrapper)

            ast = wrapper.load_doc_config()

            if additional_doc_keys:
                for doc_key in additional_doc_keys:
                    ast.add_task_info(doc_key)

            if meta and os.path.exists('dexy.rst'):
                for doc_key in DOC_KEYS:
                    ast.add_task_info(doc_key)
                    for task in ast.lookup_table.keys():
                        if 'jinja' in doc_key and not 'jinja' in task:
                            ast.add_dependency(doc_key, task)

            wrapper.batch.load_ast(ast)

            try:
                wrapper.batch.run()
                wrapper.save_db()
            except:
                raise Exception("pushd %s" % os.path.abspath("."))

            yield(wrapper.batch)

    @classmethod
    def validate(klass):
        """
        Runs dexy and validates filter list.
        """
        for batch in klass.dexy(False):
            filters_used = batch.filters_used()

        for f in klass.FILTERS_USED:
            assert f in filters_used, "filter %s not used by %s" % (f, klass.__name__)

        for f in filters_used:
            if not f in klass.FILTERS_USED:
                print "filter %s used by %s but not listed in klass.FILTERS_USED, adjust list to:" % (f, klass.__name__)
                print "   FILTERS_USED = [%s]" % ", ".join("'%s'" % f for f in filters_used)

        return True



from dexy.filter import DexyFilter
import dexy.exceptions

class Deprecated(DexyFilter):
    """
    Base class for deprecated filters.
    """
    ALIASES = []

    def process(self):
        msg = "%s\n%s" % (self.artifact.key, self.__doc__)
        raise dexy.exceptions.UserFeedback(msg)

class FilenameFilter(Deprecated):
    """
    The filename (fn) filter has been removed from dexy. Dexy should now
    automatically detect new files that are created by your scripts. You should
    remove '|fn' from your config and anywhere documents are referenced, and
    remove the 'dexy--' prefix from filenames in your scripts.
    """
    ALIASES = ['fn']




from StringIO import StringIO
from dexy.tests.utils import tempdir
from dexy.tests.utils import wrap
from dexy.version import DEXY_VERSION
from mock import patch
from nose.exc import SkipTest
import dexy.commands
import os
import sys

def test_init_wrapper():
    with tempdir():
        with open("dexy.conf", "w") as f:
            f.write("artifactsdir: custom")

        modargs = {}
        wrapper = dexy.commands.init_wrapper(modargs)
        assert wrapper.artifacts_dir == 'custom'

@patch.object(sys, 'argv', ['dexy', 'setup'])
def test_setup_with_dexy_conf_file():
    with tempdir():
        with open("dexy.conf", "w") as f:
            f.write("artifactsdir: custom")

        dexy.commands.run()
        assert os.path.exists("custom")
        assert os.path.isdir("custom")
        assert not os.path.exists("artifacts")

@patch.object(sys, 'argv', ['dexy', 'grep', '-expr', 'hello'])
def test_grep():
    with wrap() as wrapper:
        wrapper.setup_dexy_dirs()
        dexy.commands.run()

@patch.object(sys, 'argv', ['dexy', 'grep'])
@patch('sys.stdout', new_callable=StringIO)
def test_grep_without_expr(stdout):
    try:
        dexy.commands.run()
    except SystemExit as e:
        assert e.message == 1
        assert 'Option -expr' in stdout.getvalue()

def test_help_text():
    assert "Available commands" in dexy.commands.help_text()

@patch.object(sys, 'argv', ['dexy'])
@patch('sys.stderr', new_callable=StringIO)
def test_run_with_userfeedback_exception(stderr):
    with tempdir():
        with open("docs.txt", "w") as f:
            f.write("*.py|py")

        with open("hello.py", "w") as f:
            f.write("raise")

        try:
            dexy.commands.run()
            assert False, 'should raise SystemExit'
        except SystemExit as e:
            assert e.message == 1
            assert "Dexy is stopping" in stderr.getvalue()

@patch.object(sys, 'argv', ['dexy', 'invalid'])
@patch('sys.stdout', new_callable=StringIO)
def test_run_invalid_command(stdout):
    try:
        dexy.commands.run()
        assert False, 'should raise SystemExit'
    except SystemExit as e:
        assert e.message == 1

@patch.object(sys, 'argv', ['dexy', '--help'])
@patch('sys.stdout', new_callable=StringIO)
def test_run_help_old_syntax(stdout):
    dexy.commands.run()
    assert "Available commands for dexy are:" in stdout.getvalue()

@patch.object(sys, 'argv', ['dexy', '--version'])
@patch('sys.stdout', new_callable=StringIO)
def test_run_version_old_syntax(stdout):
    dexy.commands.run()
    assert DEXY_VERSION in stdout.getvalue()

@patch.object(sys, 'argv', ['dexy', 'help'])
@patch('sys.stdout', new_callable=StringIO)
def test_run_help(stdout):
    dexy.commands.run()
    assert "Available commands for dexy are:" in stdout.getvalue()

@patch.object(sys, 'argv', ['dexy', 'version'])
@patch('sys.stdout', new_callable=StringIO)
def test_run_version(stdout):
    dexy.commands.run()
    assert DEXY_VERSION in stdout.getvalue()

@patch.object(sys, 'argv', ['dexy'])
@patch('sys.stdout', new_callable=StringIO)
def test_run_dexy(stdout):
    with tempdir():
        os.makedirs('logs')
        os.makedirs('artifacts')
        dexy.commands.run()

    assert "finished in" in stdout.getvalue()

@patch.object(sys, 'argv', ['dexy', 'viewer:ping'])
@patch('sys.stdout', new_callable=StringIO)
def test_viewer_command(stdout):
    try:
        dexy.commands.run()
        assert "pong" in stdout.getvalue()
    except SystemExit:
        raise SkipTest

@patch.object(sys, 'argv', ['dexy', 'conf'])
@patch('sys.stdout', new_callable=StringIO)
def test_conf_command(stdout):
    with tempdir():
        dexy.commands.run()
        assert os.path.exists("dexy.conf")
        assert "has been written" in stdout.getvalue()

@patch.object(sys, 'argv', ['dexy', 'conf'])
@patch('sys.stdout', new_callable=StringIO)
def test_conf_command_if_path_exists(stdout):
    with tempdir():
        with open("dexy.conf", "w") as f:
            f.write("foo")
        assert os.path.exists("dexy.conf")
        dexy.commands.run()
        assert "dexy.conf already exists" in stdout.getvalue()
        assert "artifactsdir" in stdout.getvalue()

@patch.object(sys, 'argv', ['dexy', 'conf', '-p'])
@patch('sys.stdout', new_callable=StringIO)
def test_conf_command_with_print_option(stdout):
    with tempdir():
        dexy.commands.run()
        assert not os.path.exists("dexy.conf")
        assert "artifactsdir" in stdout.getvalue()

def test_filters_text():
    text = dexy.commands.filters_text()
    assert "pyg, pygments : Apply Pygments syntax highlighting." in text

def test_filters_text_single_alias():
    text = dexy.commands.filters_text(alias="pyg")
    assert "pyg, pygments" in text

def test_filters_text_versions():
    text = dexy.commands.filters_text(versions=True)
    assert "Installed version: Python" in text

def test_filters_text_single_alias_source():
    text = dexy.commands.filters_text(alias="pyg", source=True)
    assert "pyg, pygments" in text
    assert "class" in text
    assert "PygmentsFilter" in text
    assert not "class PygmentsFilter" in text

def test_filters_text_single_alias_source_nocolor():
    text = dexy.commands.filters_text(alias="pyg", source=True, nocolor=True)
    assert "pyg, pygments" in text
    assert "class PygmentsFilter" in text



from dexy.common import OrderedDict
from dexy.filter import Filter
import dexy.exceptions
import fnmatch
import json
import os
import platform
import subprocess

class SubprocessFilter(Filter):
    """
    Parent class for all filters which use the subprocess module to run external programs.
    """
    ADD_NEW_FILES = False # whether to add new files by defaut
    ALIASES = []
    CHECK_RETURN_CODE = True
    ENV = None
    INITIAL_TIMEOUT = None
    PATH_EXTENSIONS = []
    REQUIRED_EXECUTABLES = []
    TIMEOUT = None
    VERSION_COMMAND = None
    WALK_WORKING_DIRECTORY = False
    WINDOWS_VERSION_COMMAND = None
    WRITE_STDERR_TO_STDOUT = True

    @classmethod
    def executables(self):
        if platform.system() == 'Windows' and hasattr(self, 'WINDOWS_EXECUTABLE'):
            return [self.WINDOWS_EXECUTABLE]
        else:
            if hasattr(self, 'EXECUTABLE'):
                if not isinstance(self.EXECUTABLE, basestring):
                    msg = "Executable must be a string, not a %s. '%s'"
                    args = (self.EXECUTABLE.__class__.__name__, self.EXECUTABLE)
                    raise dexy.exceptions.InternalDexyProblem(msg%args)
                return [self.EXECUTABLE]
            elif hasattr(self, 'EXECUTABLES'):
                return self.EXECUTABLES

    @classmethod
    def executable(self):
        """
        Returns the executable to use, or None if no executable found on the system.
        """
        for exe in self.executables():
            if exe:
                cmd = exe.split()[0] # remove any --arguments
                if dexy.utils.command_exists(cmd):
                    return exe

    @classmethod
    def required_executables_present(klass):
        return all(dexy.utils.command_exists(exe) for exe in klass.REQUIRED_EXECUTABLES)

    @classmethod
    def is_active(klass):
        return klass.executable() and klass.required_executables_present()

    @classmethod
    def version_command(klass):
        if platform.system() == 'Windows':
            return klass.WINDOWS_VERSION_COMMAND or klass.VERSION_COMMAND
        else:
            return klass.VERSION_COMMAND

    @classmethod
    def version(klass):
        command = klass.version_command()
        if command:
            proc = subprocess.Popen(
                       command,
                       shell=True,
                       stdout=subprocess.PIPE,
                       stderr=subprocess.STDOUT
                   )

            stdout, stderr = proc.communicate()
            if proc.returncode > 0:
                return False
            else:
                return stdout.strip().split("\n")[0]

    def process(self):
        command = self.command_string()
        proc, stdout = self.run_command(command, self.setup_env())
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
        self.copy_canonical_file()

        if self.do_add_new_files():
            self.log.debug("adding new files found in %s for %s" % (self.artifact.tmp_dir(), self.artifact.key))
            self.add_new_files()

    def command_line_args(self):
        return self.args().get('args') or self.args().get('clargs')

    def command_line_scriptargs(self):
        return self.args().get('scriptargs')

    def command_string_stdout(self):
        clargs = self.command_line_args() or ''
        self.log.debug("command line args specified by user are '%s'" % clargs)
        args = {
            'prog' : self.executable(),
            'args' : clargs,
            'scriptargs' : self.command_line_scriptargs() or "",
            'script_file' : self.input_filename()
        }
        return """%(prog)s %(args)s "%(script_file)s" %(scriptargs)s""" % args

    def command_string(self):
        args = {
            'prog' : self.executable(),
            'args' : self.command_line_args() or "",
            'script_file' : self.input_filename(),
            'scriptargs' : self.command_line_scriptargs() or "",
            'output_file' : self.output_filename()
        }
        return """%(prog)s %(args)s "%(script_file)s" %(scriptargs)s "%(output_file)s" """ % args

    def ignore_nonzero_exit(self):
        return self.artifact.wrapper.ignore_nonzero_exit

    def clear_cache(self):
        self.output().clear_cache()

    def handle_subprocess_proc_return(self, command, exitcode, stderr):
        if exitcode is None:
            raise dexy.exceptions.InternalDexyProblem("no return code, proc not finished!")
        elif exitcode != 0 and self.CHECK_RETURN_CODE:
            if self.ignore_nonzero_exit():
                self.artifact.log.warn("Nonzero exit status %s" % exitcode)
                self.artifact.log.warn("output from process: %s" % stderr)
            else:
                err_msg = "The command '%s' for %s exited with nonzero exit status %s." % (command, self.artifact.key, exitcode)
                if stderr:
                    err_msg += " Here is stderr:\n%s" % stderr
                self.output().clear_cache()
                raise dexy.exceptions.UserFeedback(err_msg)

    def setup_timeout(self):
        return self.args().get('timeout', self.TIMEOUT)

    def setup_initial_timeout(self):
        return self.args().get('initial_timeout', self.INITIAL_TIMEOUT)

    def setup_env(self):
        env = os.environ

        # Add parameters set in class's ENV variable.
        if self.ENV:
            env.update(self.ENV)

        # Add parameters set in filter arguments.
        env.update(self.args().get('env', {}))

        # Add parameters in wrapper's env dict
        for doc in self.processed():
            if doc.final_artifact.ext == '.json':
                for env_item, env_value in doc.output().json_as_dict().iteritems():
                    if env_item.startswith("DEXY_"):
                        env[env_item] = env_value

        # Add any path extensions to PATH
        if self.PATH_EXTENSIONS:
            paths = [env['PATH']] + self.PATH_EXTENSIONS
            env['PATH'] = ":".join(paths)

        return env

    def add_new_files(self):
        wd = self.artifact.tmp_dir()

        do_add_new = self.do_add_new_files()

        for dirpath, dirnames, filenames in os.walk(wd):
            for filename in filenames:
                filepath = os.path.join(dirpath, filename)
                filesize = os.path.getsize(filepath)
                relpath = os.path.relpath(filepath, wd)

                already_have_file = (relpath in self.artifact.wrapper.batch.doc_names())
                empty_file = (filesize == 0)

                if isinstance(do_add_new, list):
                    is_valid_file_extension = False
                    for pattern in do_add_new:
                        if "*" in pattern:
                            if fnmatch.fnmatch(relpath, pattern):
                                is_valid_file_extension = True
                                continue
                        else:
                            if filename.endswith(pattern):
                                is_valid_file_extension = True
                                continue
                elif isinstance(do_add_new, basestring):
                    is_valid_file_extension = False
                    for pattern in [do_add_new]:
                        if "*" in pattern:
                            if fnmatch.fnmatch(relpath, pattern):
                                is_valid_file_extension = True
                                continue
                        else:
                            if filename.endswith(pattern):
                                is_valid_file_extension = True
                                continue
                elif isinstance(do_add_new, bool):
                    if not do_add_new:
                        raise dexy.exceptions.InternalDexyProblem("should not get here")
                    is_valid_file_extension = True
                else:
                    raise dexy.exceptions.InternalDexyProblem("type is %s value is %s" % (do_add_new.__class__, do_add_new))

                if (not already_have_file) and is_valid_file_extension and (not empty_file):
                    with open(filepath, 'rb') as f:
                        contents = f.read()
                    self.add_doc(relpath, contents)

    def do_walk_working_directory(self):
        if self.args().has_key('walk-working-dir'):
            return self.args()['walk-working-dir']
        else:
            return self.WALK_WORKING_DIRECTORY

    def walk_working_directory(self, doc=None, section_name=None):
        if not doc:
            if section_name:
                doc_key = "%s-%s-files" % (self.output().long_name(), section_name)
            else:
                doc_key = "%s-files" % self.output().long_name()

            doc = self.add_doc(doc_key, {})

        wd = self.artifact.tmp_dir()
        for dirpath, dirnames, filenames in os.walk(wd):
            for filename in filenames:
                filepath = os.path.join(dirpath, filename)
                relpath = os.path.relpath(filepath, wd)

                with open(filepath, "rb") as f:
                    contents = f.read()
                try:
                    json.dumps(contents)
                    doc.output().append(relpath, contents)
                except UnicodeDecodeError:
                    doc.output().append(relpath, 'binary')

        return doc

    def write_stderr_to_stdout(self):
        # TODO allow customizing this in args
        return self.WRITE_STDERR_TO_STDOUT

    def run_command(self, command, env, input_text=None):
        wd = self.setup_wd()

        stdout = subprocess.PIPE

        if input_text:
            stdin = subprocess.PIPE
        else:
            stdin = None

        if self.write_stderr_to_stdout():
            stderr = stdout
        else:
            stderr = subprocess.PIPE

        self.log.debug("about to run '%s' in '%s'" % (command, wd))
        proc = subprocess.Popen(command, shell=True,
                                    cwd=wd,
                                    stdin=stdin,
                                    stdout=stdout,
                                    stderr=stderr,
                                    env=env)

        if input_text:
            self.log.debug("about to send input_text '%s'" % input_text)

        stdout, stderr = proc.communicate(input_text)
        self.log.debug(u"stdout is '%s'" % stdout.decode('utf-8'))
        self.log.debug(u"stderr is '%s'" % stderr.decode('utf-8'))

        return (proc, stdout)

    def copy_canonical_file(self):
        canonical_file = os.path.join(self.artifact.tmp_dir(), self.output().name)
        if not self.output().is_cached() and os.path.exists(canonical_file):
            self.output().copy_from_file(canonical_file)

class SubprocessStdoutFilter(SubprocessFilter):
    """
    Subclass of SubprocessFilter which runs a command and returns the stdout generated by that command as its output.
    """
    WRITE_STDERR_TO_STDOUT = False
    REQUIRE_OUTPUT = False

    def process(self):
        command = self.command_string_stdout()
        proc, stdout = self.run_command(command, self.setup_env())
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
        self.output().set_data(stdout)

        if self.do_walk_working_directory():
            self.walk_working_directory()

        if self.do_add_new_files():
            self.log.debug("adding new files found in %s for %s" % (self.artifact.tmp_dir(), self.artifact.key))
            self.add_new_files()

class SubprocessCompileFilter(SubprocessFilter):
    """
    Base class for filters which need to compile code, then run the compiled executable.
    """
    ADD_NEW_FILES = True
    COMPILED_EXTENSION = ".o"
    CHECK_RETURN_CODE = False # Whether to check return code when running compiled executable.
    EXECUTABLES = []

    def compile_command_string(self):
        wf = os.path.basename(self.input().name)
        of = self.compiled_filename()
        compiler_args = self.args().get("compiler-args", "")
        return "%s %s %s -o %s" % (self.executable(), compiler_args, wf, of)

    def compiled_filename(self):
        basename = os.path.basename(self.input().name)
        nameroot = os.path.splitext(basename)[0]
        return "%s%s" % (nameroot, self.COMPILED_EXTENSION)

    def run_command_string(self):
        return "./%s" % self.compiled_filename()

    def process(self):
        env = self.setup_env()

        # Compile the code
        command = self.compile_command_string()
        proc, stdout = self.run_command(command, env)

        # test exitcode from the *compiler*
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)

        # Run the compiled code
        command = self.run_command_string()
        proc, stdout = self.run_command(command, env)

        # This tests exitcode from the compiled script.
        if self.CHECK_RETURN_CODE:
            self.handle_subprocess_proc_return(command, proc.returncode, stdout)

        self.output().set_data(stdout)

        if self.do_add_new_files():
            self.log.debug("adding new files found in %s for %s" % (self.artifact.tmp_dir(), self.artifact.key))
            self.add_new_files()

class SubprocessInputFilter(SubprocessFilter):
    """
    Filters which run a task in subprocess while also writing content to stdin for that process.
    """
    CHECK_RETURN_CODE = False
    WRITE_STDERR_TO_STDOUT = False
    OUTPUT_DATA_TYPE = 'sectioned'

    def process(self):
        command = self.command_string()

        inputs = list(self.artifact.doc.node.walk_input_docs())

        output = OrderedDict()

        if len(inputs) == 1:
            doc = inputs[0]
            for section_name, section_text in doc.output().as_sectioned().iteritems():
                proc, stdout = self.run_command(command, self.setup_env(), section_text)
                if self.CHECK_RETURN_CODE:
                    self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                output[section_name] = stdout
        else:
            for doc in inputs:
                proc, stdout = self.run_command(command, self.setup_env(), unicode(doc.output()))
                if self.CHECK_RETURN_CODE:
                    self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                output[doc.key] = stdout

        self.output().set_data(output)

class SubprocessInputFileFilter(SubprocessFilter):
    """
    Filters which run one or more input files through the script via filenames.
    """
    CHECK_RETURN_CODE = False
    WRITE_STDERR_TO_STDOUT = False
    OUTPUT_DATA_TYPE = 'sectioned'

    def command_string_for_input(self, input_doc):
        return "%s %s %s" % (self.executable(), self.input_filename(), input_doc.output().name)

    def process(self):
        inputs = list(self.artifact.doc.node.walk_input_docs())

        output = OrderedDict()

        for doc in inputs:
            command = self.command_string_for_input(doc)
            proc, stdout = self.run_command(command, self.setup_env())
            if self.CHECK_RETURN_CODE:
                self.handle_subprocess_proc_return(command, proc.returncode, stdout)
            output[doc.key] = stdout

        self.output().set_data(output)

class SubprocessCompileInputFilter(SubprocessCompileFilter):
    """
    Filters which compile code, then run it with input.
    """
    CHECK_RETURN_CODE = False
    WRITE_STDERR_TO_STDOUT = False
    OUTPUT_DATA_TYPE = 'sectioned'

    def process(self):
        # Compile the code
        command = self.compile_command_string()
        proc, stdout = self.run_command(command, self.setup_env())
        self.handle_subprocess_proc_return(command, proc.returncode, stdout)

        command = self.run_command_string()

        inputs = list(self.artifact.doc.node.walk_input_docs())

        output = OrderedDict()

        if len(inputs) == 1:
            doc = inputs[0]
            for section_name, section_text in doc.output().as_sectioned().iteritems():
                proc, stdout = self.run_command(command, self.setup_env(), section_text)
                if self.CHECK_RETURN_CODE:
                    self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                output[section_name] = stdout
        else:
            for doc in inputs:
                proc, stdout = self.run_command(command, self.setup_env(), doc.output().as_text())
                if self.CHECK_RETURN_CODE:
                    self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                output[doc.key] = stdout

        self.output().set_data(output)



from dexy.artifact import InitialVirtualArtifact, FilterArtifact
from dexy.doc import Doc
from dexy.node import DocNode
from dexy.tests.utils import assert_output
from dexy.tests.utils import runfilter
from dexy.tests.utils import wrap

def test_process_text_filter():
    assert_output("processtext", "hello", "Dexy processed the text 'hello'")

def test_process_text_to_dict_filter():
    assert_output("processtexttodict", "hello", {'1': "Dexy processed the text 'hello'"})

def test_process_dict_filter():
    assert_output("processdict", {'1': 'hello'}, {'1': "Dexy processed the text 'hello'"})

def test_process_method():
    assert_output("process", "hello", "Dexy processed the text 'hello'")

def test_process_method_manual_write():
    assert_output("processmanual", "hello", "Dexy processed the text 'hello'")

def test_process_method_with_dict():
    assert_output("processwithdict", "hello", {'1' : "Dexy processed the text 'hello'"})

def test_add_new_document():
    with runfilter("newdoc", "hello") as doc:
        assert doc.node.inputs[-1].key == "subdir/newfile.txt|processtext"
        assert doc.output().data() == "we added a new file"
        assert isinstance(doc.node.inputs[-1].children[0], Doc)
        assert doc.node.inputs[-1].children[0].output().data() == "Dexy processed the text 'newfile'"

        assert "Doc:subdir/example.txt|newdoc" in doc.wrapper.batch.lookup_table
        assert "Doc:subdir/newfile.txt|processtext" in doc.wrapper.batch.lookup_table

def test_key_value_example():
    with wrap() as wrapper:
        doc = Doc(
                "hello.txt|keyvalueexample",
                contents="hello",
                wrapper=wrapper
                )

        wrapper.run_docs(doc)

        assert doc.output().as_text() == "foo: bar"

def test_access_other_documents():
    with wrap() as wrapper:
        node = DocNode("hello.txt|newdoc", contents="hello", wrapper=wrapper)
        parent = DocNode("test.txt|others",
                    inputs=[node],
                    contents="hello",
                    wrapper=wrapper)
        wrapper.run_docs(parent)

        expected_items = [
            "Here is a list of previous docs in this tree (not including test.txt|others).",
            "newfile.txt|processtext (2 children, 0 inputs, length 33)",
            "hello.txt|newdoc (2 children, 0 inputs, length 19)"
            ]

        output = unicode(parent.children[-1].output())

        for item in expected_items:
            assert item in output

def test_doc_children_artifacts():
    with wrap() as wrapper:
        node = DocNode("hello.txt|newdoc", contents="hello", wrapper=wrapper)

        parent = DocNode("parent.txt|process",
                inputs=[node],
                contents="hello",
                wrapper=wrapper)

        wrapper.root_nodes = [parent]

        node.populate()
        parent.populate()

        doc = node.children[0]

        # Doc's children are artifacts
        assert len(doc.children) == 2
        assert isinstance(doc.children[0], InitialVirtualArtifact)
        assert isinstance(doc.children[1], FilterArtifact)

        # Parent's children and inputs are other nodes
        assert len(parent.children) == 1
        assert len(parent.inputs) == 1

        assert isinstance(parent.children[0], Doc)
        assert isinstance(parent.inputs[0], DocNode)

        node = DocNode("hello.txt|newdoc", contents="hello", wrapper=wrapper)

        parent = DocNode("parent.txt|process",
                inputs=[node],
                contents="hello",
                wrapper=wrapper)

        wrapper.run_docs(parent)

        doc = node.children[0]

        assert len(doc.children) == 2
        assert isinstance(doc.children[0], InitialVirtualArtifact)
        assert isinstance(doc.children[1], FilterArtifact)

        assert len(parent.inputs) == 1
        assert len(parent.children) == 1

        assert "Doc:hello.txt|newdoc" in wrapper.batch.lookup_table
        assert "Doc:parent.txt|process" in wrapper.batch.lookup_table
        assert "Doc:newfile.txt|processtext" in wrapper.batch.lookup_table



from dexy.common import OrderedDict
from dexy.plugin import PluginMeta
import hashlib

class Metadata:
    ALIASES = []
    __metaclass__ = PluginMeta

    @classmethod
    def is_active(klass):
        return True

    def get_string_for_hash(self):
        ordered = OrderedDict()
        for k in sorted(self.__dict__):
            ordered[k] = str(self.__dict__[k])

        return str(ordered)

class Md5(Metadata):
    """
    Class that stores metadata for a task. Uses md5 to calculate hash.
    """
    ALIASES = ['md5']
    def compute_hash(self):
        text = self.get_string_for_hash()
        return hashlib.md5(text).hexdigest()



from dexy.filter import DexyFilter
import json

try:
    import nltk
    AVAILABLE = True
except ImportError:
    AVAILABLE = False

class NltkFilter(DexyFilter):
    """
    Base class for NLTK text processing filters. Returns a JSON object with
    various metadata about the input text. Just a demo of what's possible
    rather than useful for now. See http://dexy.it/gallery/examples/nltk/

    """
    ALIASES = ["nltk"]
    INPUT_EXTENSIONS = [".txt"]
    OUTPUT_EXTENSIONS = [".json"]

    @classmethod
    def is_active(klass):
        return AVAILABLE

    def process_text(self, input_text):
        data = {}
        tokens = nltk.word_tokenize(input_text)

        data['wc'] = len(tokens)

        bigram_measures = nltk.collocations.BigramAssocMeasures()
        finder = nltk.collocations.BigramCollocationFinder.from_words(tokens)
        data['collocations'] = finder.nbest(bigram_measures.pmi, 10)

        nltk.download("maxent_treebank_pos_tagger")
        data['pos-tags'] = nltk.pos_tag(tokens)

        return json.dumps(data)



from dexy.tests.utils import wrap
from dexy.node import DocNode

def test_split_html_filter():
    with wrap() as wrapper:
        contents="""
        <p>This is at the top.</p>
        <!-- split "a-page" -->
        some content on a page
        <!-- split "another-page" -->
        some content on another page
        <!-- endsplit -->
        bottom
        """

        node = DocNode("subdir/example.html|splithtml", contents=contents, wrapper=wrapper)
        wrapper.run_docs(node)

        assert node.inputs[0].key == "subdir/a-page.html"
        assert node.inputs[1].key == "subdir/another-page.html"

        doc = node.children[0]
        od = doc.output().data()

        assert "<p>This is at the top.</p>" in od
        assert '<a href="a-page.html">' in od
        assert '<a href="another-page.html">' in od
        assert "bottom" in od

        assert "<p>This is at the top.</p>" in node.inputs[0].children[0].output().data()
        assert "some content on a page" in node.inputs[0].children[0].output().data()
        assert "bottom" in node.inputs[0].children[0].output().data()

        assert "<p>This is at the top.</p>" in node.inputs[1].children[0].output().data()
        assert "some content on another page" in node.inputs[1].children[0].output().data()
        assert "bottom" in node.inputs[1].children[0].output().data()

def test_split_html_additional_filters():
    with wrap() as wrapper:
        contents="""
        <p>This is at the top.</p>
        <!-- split "a-page" -->
        some content on a page
        <!-- split "another-page" -->
        some content on another page
        <!-- endsplit -->
        bottom
        """

        node = DocNode("example.html|splithtml",
                contents=contents,
                splithtml = { "keep-originals" : False, "additional-doc-filters" : "processtext" },
                wrapper=wrapper
              )
        wrapper.run_docs(node)

        doc = node.children[0]

        assert node.inputs[0].key == "a-page.html|processtext"
        assert node.inputs[1].key == "another-page.html|processtext"

        od = doc.output().data()
        assert "<p>This is at the top.</p>" in od
        assert '<a href="a-page.html">' in od
        assert '<a href="another-page.html">' in od
        assert "bottom" in od

        a_page = node.inputs[0].children[0]
        a_page_data = a_page.output().data()
        assert "<p>This is at the top.</p>" in a_page_data
        assert "some content on a page" in a_page_data
        assert "bottom" in a_page_data
        assert "Dexy processed the text" in a_page_data

        another_page = node.inputs[1].children[0]
        another_page_data = another_page.output().data()
        assert "<p>This is at the top.</p>" in another_page_data
        assert "some content on another page" in another_page_data
        assert "bottom" in another_page_data
        assert "Dexy processed the text" in another_page_data
